/* Copyright 2009-2016 David Hadka
 *
 * This file is part of the MOEA Framework.
 *
 * The MOEA Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * The MOEA Framework is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the MOEA Framework.  If not, see <http://www.gnu.org/licenses/>.
 */
import java.io.File;
import java.io.IOException;
import java.time.Duration;
import java.time.LocalDateTime;

import org.moeaframework.Analyzer;
import org.moeaframework.Executor;
import org.moeaframework.analysis.plot.Plot;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.Population;
import org.moeaframework.core.PopulationIO;
import org.moeaframework.core.Solution;
import org.moeaframework.core.spi.ProblemFactory;
//import org.moeaframework.problem.DTLZ.mydtlz;
import org.moeaframework.problem.misc.DENT;

/**
 * Demonstrates using an Executor to solve the UF1 test problem with NSGA-II,
 * one of the most widely-used multiobjective evolutionary algorithms.
 */
public class Example1 {

	public static void main(String[] args) throws IOException {
		//configure and run this experiment////
		LocalDateTime BeforeRun = LocalDateTime.now();

		String AlgorithmName="T-NSGA-II";	
//		String AlgorithmName="T-R2-EMOA";
//		String AlgorithmName="T-SMS-EMOA";
//		String AlgorithmName="R-NSGA-II";
//		String AlgorithmName="DF-SMS-EMOA";
//		String AlgorithmName="AS-EMOA";
//		String AlgorithmName="SMS-EMOA";
//		String AlgorithmName="R2-EMOA";
//		String AlgorithmName="NSGA-II";
//		String AlgorithmName="NSGA-III";
//		String AlgorithmName="RVEA";
//       String AlgorithmName="T-MOEA/D";
//      String AlgorithmName="R2-EMOA";
//		String AlgorithmName="NSGA-III";
       String ProblemName="DTLZ2_3";
//       String ProblemName="DTLZ1_3";    
       
		NondominatedPopulation result = new Executor()
				//.withProblemClass(myProblem.class)
				.withProblem(ProblemName)
//			.withProblemClass(DENT.class)
				.withProperty("populationSize", 100)
				// use elapsed time as termination criterion
//				.withProperty("maxTime", 1000)
//				.withProperty("sbx rate",0.9)
				//.withProblemClass(mydtlz.class)
//				.withProperty("pm.rate", 0.5)//ZDT4
				.withAlgorithm(AlgorithmName)
				.withMaxEvaluations(25000)
				.run();
		
		


//		
		Analyzer analyzer = new Analyzer()
				.withProblem(ProblemName)
//				.includeHypervolume();
//				.withIdealPoint(minBound)
//				.withReferencePoint(maxBound)
//				.includeRPBasedConvergence()
//				.includeRPBasedConvergenceChebycheff()
				.includePreferenceBasedHypervolume();
//				.includeRHypervolume();
//		        .includePreferenceBasedHypervolumeChebyshev();
//		        .includePopulationSizeInROI();

		analyzer.add(AlgorithmName, result);
//		analyzer.printAnalysis();
		
		//display the results
	//	System.out.format("Objective1  Objective2  Objective3 %n");
//		System.out.format("Objective1  Objective2  %n");
//		for (Solution solution : result) {
//			System.out.format("%.4f      %.4f     %n",
//					solution.getObjective(0),
//					solution.getObjective(1)
//					);
//		}
		LocalDateTime AfterRun = LocalDateTime.now();
		Duration duration = Duration.between(BeforeRun, AfterRun);
		double RunningTime = duration.getSeconds();
		System.out.println("RunTime: " + RunningTime + " seconds");
		
		new Plot()
		.add(AlgorithmName, result)
		.show();
		PopulationIO.writeObjectives(new File("objectives.txt"), result);
//		PopulationIO.write(new File("solutions.txt"), result);
		
		//find closest solution in DTLZ2 reference set
//		NondominatedPopulation factorySet = null;
//		factorySet = ProblemFactory.getInstance().getReferenceSet(
//				"DTLZ2_3");
//		
//	double[] RP1=new double[]{0.2,0.2,0.6};
//	double[] RP2=new double[] {0.8,0.6,1.0};
//	//double[] solution1=FindClosestEuclideanDistanceToRP(result,RP1);
//	double[] solution1=FindClosestEuclideanDistanceToRP(factorySet,RP1);
//	double[] solution2=FindClosestEuclideanDistanceToRP(factorySet,RP2);
//
//	System.out.format("%.6f      %.6f     %.6f  %n",	solution1[0],solution1[1],solution1[2]	);
//	System.out.format("%.6f      %.6f     %.6f  %n",	solution2[0],solution2[1],solution2[2]	);

	}

	
	public static double[] FindClosestEuclideanDistanceToRP(NondominatedPopulation p,double[]RP){
		double[] solution=new double[p.get(0).getNumberOfObjectives()];
		int numberOfObjectives = p.get(0).getNumberOfObjectives();
		// array of max and min for each objective
		
			double minidis=Double.POSITIVE_INFINITY;
			for(Solution s: p){
				double dis=0;
			for (int k = 0; k < numberOfObjectives; k++) {
				double d=Math.pow(s.getObjective(k)-RP[k],2);				
				dis+=d;			
			}
			if(dis<minidis){
				minidis=dis;
				for (int k = 0; k < numberOfObjectives; k++){
					solution[k]=s.getObjective(k);
				}
			}
			
			
			}
		return solution;
		
	}
	public static double[] FindClosestChebyshevDistanceToRP(NondominatedPopulation p,double[]RP){
		double[] solution=new double[p.get(0).getNumberOfObjectives()];
		int numberOfObjectives = p.get(0).getNumberOfObjectives();
		// array of max and min for each objective	
			double minidis=Double.POSITIVE_INFINITY;
			for(Solution s: p){
				double dis=0;
			for (int k = 0; k < numberOfObjectives; k++) {
				double d=Math.abs(s.getObjective(k)-RP[k]);	
				if(d>dis)
				{
					dis=d;	
				}
					
			}
			if(dis<minidis){
				minidis=dis;
				for (int k = 0; k < numberOfObjectives; k++){
					solution[k]=s.getObjective(k);
				}
			}
			
			
			}
		return solution;
		
	}

	}


