import java.io.IOException;
import java.time.Duration;
import java.time.LocalDateTime;

import org.moeaframework.Analyzer;
import org.moeaframework.Executor;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.Solution;

public class RPBasedComparison {
	
	public static void main(String[] args) throws IOException {
		String problem = "ZDT3";
//		String[] algorithms = { "T-NSGA-II", "T-SMS-EMOA", "T-R2-EMOA", "R-NSGA-II" };
		
		String[] algorithms = { "T-NSGA-II", "R-NSGA-II"};
		
		

		//setup the experiment
		Executor executor = new Executor()
				.withProblem(problem)
				.withProperty("populationSize", 50)
//				.withProperty("pm.rate", 0.5)//ZDT4
				.withMaxEvaluations(50000);

		
		Analyzer analyzer = new Analyzer()
				.withProblem(problem)
//				.includeRPBasedConvergence()
//				.includeRPBasedConvergenceChebycheff()
				.includePreferenceBasedHypervolume()
				.includePreferenceBasedHypervolumeChebyshev()
				.includeRHypervolume()
				.showIndividualValues()
				.showStatisticalSignificance();
		
		int seed=20;

		//run each algorithm for number of seed
		for (String algorithm : algorithms) {
			LocalDateTime BeforeRun = LocalDateTime.now();
			analyzer.addAll(algorithm, 
					executor.withAlgorithm(algorithm).runSeeds(seed));
			LocalDateTime AfterRun = LocalDateTime.now();
			Duration duration = Duration.between(BeforeRun, AfterRun);
			double RunningTime = duration.getSeconds();
			System.out.println(algorithm+": " + RunningTime + " seconds\n");
			
		}

		//print the results
		analyzer.printAnalysis();
	}
	public double[] FindClosestEuclideanDistanceToRP(NondominatedPopulation p,double[]RP){
		double[] solution=new double[p.get(0).getNumberOfObjectives()];
		int numberOfObjectives = p.get(0).getNumberOfObjectives();
		// array of max and min for each objective
		
			double minidis=Double.POSITIVE_INFINITY;
			for(Solution s: p){
				double dis=0;
			for (int k = 0; k < numberOfObjectives; k++) {
				double d=Math.pow(s.getObjective(k)-RP[k],2);				
				dis+=d;			
			}
			if(dis<minidis){
				minidis=dis;
				for (int k = 0; k < numberOfObjectives; k++){
					solution[k]=s.getObjective(k);
				}
			}
			
			
			}
		return solution;
		
	}
	public double[] FindClosestChebyshevDistanceToRP(NondominatedPopulation p,double[]RP){
		double[] solution=new double[p.get(0).getNumberOfObjectives()];
		int numberOfObjectives = p.get(0).getNumberOfObjectives();
		// array of max and min for each objective	
			double minidis=Double.POSITIVE_INFINITY;
			for(Solution s: p){
				double dis=0;
			for (int k = 0; k < numberOfObjectives; k++) {
				double d=Math.abs(s.getObjective(k)-RP[k]);	
				if(d>dis)
				{
					dis=d;	
				}
					
			}
			if(dis<minidis){
				minidis=dis;
				for (int k = 0; k < numberOfObjectives; k++){
					solution[k]=s.getObjective(k);
				}
			}
			
			
			}
		return solution;
		
	}
	
	
}
