/* Copyright 2009-2016 David Hadka
 *
 * This file is part of the MOEA Framework.
 *
 * The MOEA Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * The MOEA Framework is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the MOEA Framework.  If not, see <http://www.gnu.org/licenses/>.
 */
import java.io.IOException;
import java.time.Duration;
import java.time.LocalDateTime;

import org.moeaframework.Analyzer;
import org.moeaframework.Executor;

/**
 * Demonstrates how only a few lines of code are necessary to setup, run
 * and statistically compare multiple algorithms.
 */
public class RegionBasedComparison {

	public static void main(String[] args) throws IOException {
		String problem = "ZDT1";
		String[] algorithms = { "T-NSGA-II",  "T-MOEA/D","T-SMS-EMOA", "T-R2-EMOA" };
		
//		String[] algorithms = { "T-NSGA-II", "R-NSGA-II" };

		//setup the experiment
		Executor executor = new Executor()
				.withProblem(problem)
				.withProperty("populationSize", 50)
//				.withProperty("pm.rate", 0.5)
				.withMaxEvaluations(20000);

		
		Analyzer analyzer = new Analyzer()
				.withProblem(problem)
				.includePopulationSizeInROI()
				.includePreferenceBasedHypervolume()
				.showStatisticalSignificance()
				.showIndividualValues();

		int seed=10;

		//run each algorithm for 50 seeds
		for (String algorithm : algorithms) {
			LocalDateTime BeforeRun = LocalDateTime.now();
			analyzer.addAll(algorithm, 
					executor.withAlgorithm(algorithm).runSeeds(seed));
			LocalDateTime AfterRun = LocalDateTime.now();
			Duration duration = Duration.between(BeforeRun, AfterRun);
			double RunningTime = duration.getSeconds();
			System.out.println(algorithm+": " + RunningTime + " seconds\n");
		}

		//print the results
		analyzer.printAnalysis();
	}
	
}

