package SatelliteMissionPlanning;

import java.awt.List;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;

import org.moeaframework.Analyzer;
import org.moeaframework.Executor;
import org.moeaframework.Instrumenter;
import org.moeaframework.analysis.collector.Accumulator;
import org.moeaframework.analysis.plot.Plot;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.Population;
import org.moeaframework.core.PopulationIO;
import org.moeaframework.core.Solution;
import org.moeaframework.core.indicator.IndicatorUtils;
import org.moeaframework.core.variable.EncodingUtils;
import org.moeaframework.util.Vector;


public class RuntimeAnalysis {
	public static void main(String[] args) throws IOException {
	//	File myFile = new File("2-100-Random.xlsx");
		File myFile = new File("2-100-Clustered.xlsx");
		if (myFile == null) {
			System.err.println("Unable to find the input file.");
			System.exit(-1);
		}
		LocalDateTime BeforeRun = LocalDateTime.now();
		String algorithmName="T-NSGA-II";

		SatelliteProblem satelliteProblemInstance = new SatelliteProblem(myFile);
		
//		// setup the instrumenter to record the generational distance metric
		Instrumenter instrumenter = new Instrumenter()
				.withProblem(satelliteProblemInstance)
				.withFrequency(2000)
//				.withReferenceSet(ReferenceSet)
				.attachElapsedTimeCollector()
				.attachHypervolumeInROICollector();
//		        .attachHypervolumeCollector();
//		        .attachPopulationSizeInROICollector();
//				.attachPreferenceInvertedGenerationalDistanceCollector();
		
		
		NondominatedPopulation result1 = new Executor().withProblem(satelliteProblemInstance)
				.withAlgorithm(algorithmName)
////			    .withAlgorithm("R2-EMOA")
///////				// .withAlgorithm("NSGAIII")
				.withMaxEvaluations(3000000)
				.withProperty("populationSize", 50)
				//3000000=50 min
				//900000=15min
				.withProperty("maxTime", 900000)
				.withInstrumenter(instrumenter)
				.run();
		
		  Accumulator accumulator = instrumenter.getLastAccumulator();
////		// print the runtime dynamics
//	System.out.format("  NFE    Time    IGD-CF%n");

	  // output accumulator result
		for (int i=0; i<accumulator.size("NFE"); i++) {
		//	System.out.format("%5d    %-8.4f  %-8.6f  %5d %n",
					System.out.format("%5d    %-8.4f  %-8.6f  %n",		
							accumulator.get("NFE", i),
							accumulator.get("Elapsed Time", i),
							accumulator.get("Hypervolume", i));
	//						accumulator.get("Population Size In ROI", i));
		}
		
		new Plot().add(algorithmName, result1).show();
		
		PopulationIO.writeObjectives(new File("objectives.txt"), result1);
	}

}
