package SatelliteMissionPlanning;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.time.Duration;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.Period;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.moeaframework.core.Problem;
import org.moeaframework.core.Solution;
import org.moeaframework.core.variable.EncodingUtils;
import org.moeaframework.util.Vector;

import java.util.Map;
import java.util.Random;

class AccessWindow implements Comparable<AccessWindow> {
	// read directly from input
	String SatelliteID;
	String TargetID;
	LocalDateTime EarliestStartTime;
	LocalDateTime LatestStopTime;
	double Duration;
	double Azimuth;
	double Elevation;
	double Range;
	double profit;

	// decision variable

	boolean execute;
	LocalDateTime StartTime;

	// Intermediate variables
	double Resolution;
	double ObservingAzimuth;
	double ObservingElevation;
	double ObservingRange;

	public AccessWindow(AccessWindow a) {
		super();
		this.SatelliteID = a.SatelliteID;
		this.TargetID = a.TargetID;
		this.EarliestStartTime = a.EarliestStartTime;
		this.LatestStopTime = a.LatestStopTime;
		this.Duration = a.Duration;
		this.Azimuth = a.Azimuth;
		this.Elevation = a.Elevation;
		this.Range = a.Range;
		this.execute = a.execute;
		this.StartTime = a.StartTime;

		this.Resolution = a.Resolution;
		this.ObservingAzimuth = a.ObservingAzimuth;
		this.ObservingElevation = a.ObservingElevation;
		this.ObservingRange = a.ObservingRange;

	}

	public AccessWindow() {

	}

	@Override
	public int compareTo(AccessWindow o1) {
		
		
		if(EarliestStartTime.equals(o1.EarliestStartTime))
		{
			return 0;
		}
		else{
			int k=EarliestStartTime.compareTo(o1.EarliestStartTime);	
			return 	k;		

		}
		
	}

}

class WindowAndIndex {
	AccessWindow window;
	int index;

}

public class SatelliteProblem implements Problem {

	private static int NumberOfSatellite;
	private ArrayList<String> Satellites;
	private int NumberOfInputTarget;
	private int TotalProfitInput;
	private ArrayList<String> Targets;

	private ArrayList<AccessWindow> InputAccessWindow;
	private static ArrayList<AccessWindow> AccessWindowList;
	private int NumberOfAccessWindow;
	private static double MinimalObservationInverval = 90; // minimal time
															// interval between
															// two observation
															// of one satellite,
															// second
	private static double MinimalObservationDuration = 3; // minimal time
															// duration of one
															// observation,
															// second
	public static double NANOTOSEC = 1000000000;
	private LocalDateTime EarliestStartTime; // earliest start time of the first
												// window of window list
	private LocalDateTime LatestEndTime; // Latest end time of the last window
											// of window list
	private Duration TotalDuration;
	private static Map<String, ArrayList<AccessWindow>> GroupByTarget;// Access windows
																// group by
																// target

	public SatelliteProblem(File myfile) throws IOException {
		super();

		load(myfile);
	}

	// transfer Date to LocalDateTime
	public LocalDateTime DateToDateTime(Date date) {
		Instant instant = date.toInstant();
		ZoneId zone = ZoneId.systemDefault();
		LocalDateTime localDateTime1 = LocalDateTime.ofInstant(instant, zone);
		return localDateTime1;
	}

	// get the earlier LocalDateTime
	public LocalDateTime GetEarlier(LocalDateTime time1, LocalDateTime time2) {
		LocalDateTime Earlier = time1;
		if (time1.isAfter(time2))
			Earlier = time2;

		return Earlier;
	}

	// get the later LocalDateTime
	public static LocalDateTime GetLater(LocalDateTime time1, LocalDateTime time2) {
		LocalDateTime Earlier = time1;
		if (time1.isBefore(time2))
			Earlier = time2;

		return Earlier;
	}

	private void load(File file) throws IOException {
		InputAccessWindow = new ArrayList<AccessWindow>();
		Satellites = new ArrayList<String>();
		Targets = new ArrayList<String>();

		XSSFWorkbook myWorkBook = new XSSFWorkbook(new FileInputStream(file));

		XSSFSheet mySheet = myWorkBook.getSheetAt(0);

		System.out.println("Going to read .xlsx file!");

		Row row;
		for (int i = 1; i < mySheet.getPhysicalNumberOfRows(); i++) {
			row = mySheet.getRow(i);

			AccessWindow tempwindow = new AccessWindow();
			// satellite ID
			Cell c1 = row.getCell(0);
			String satelliteID = c1.getStringCellValue();
			if (!Satellites.contains(satelliteID))
				Satellites.add(satelliteID);

			tempwindow.SatelliteID = satelliteID;
			// target ID
			Cell c2 = row.getCell(1);
			String targetID = c2.getStringCellValue();

			// Profit
			Cell c9 = row.getCell(8);
			Double profit = c9.getNumericCellValue();
			tempwindow.profit = profit;

			if (!Targets.contains(targetID)) {
				Targets.add(targetID);
				TotalProfitInput += profit;
			}

			tempwindow.TargetID = targetID;
			// EarliestStartTime
			Cell c3 = row.getCell(2);
			Date dateEST = c3.getDateCellValue();
			LocalDateTime localEST = DateToDateTime(dateEST);
			tempwindow.EarliestStartTime = localEST;
			// LatestStopTime
			Cell c4 = row.getCell(3);
			Date dateLET = c4.getDateCellValue();
			LocalDateTime LocalLET = DateToDateTime(dateLET);
			tempwindow.LatestStopTime = LocalLET;
			// Duration
			Cell c5 = row.getCell(4);
			Double dou = c5.getNumericCellValue();
			// Duration dur=Duration.ofMillis((long) (dou*1000));
			tempwindow.Duration = dou;
			// Azimuth
			Cell c6 = row.getCell(5);
			if (c6 != null) {
				Double azimuth = c6.getNumericCellValue();

				tempwindow.Azimuth = azimuth;
			} else {
				tempwindow.Azimuth = 0;
			}
			// Elevation
			Cell c7 = row.getCell(6);
			if (c7 != null) {
				Double elevation = c7.getNumericCellValue();
				tempwindow.Elevation = elevation;
			} else {
				tempwindow.Elevation = 0;
			}
			// Range
			Cell c8 = row.getCell(7);
			if (c8 != null) {
				Double Range = c8.getNumericCellValue();
				tempwindow.Range = Range;
			} else {
				tempwindow.Range = 0;
			}

			// calculate start time at best start time
			LocalDateTime BestStartTime = StartTimeTransformation(tempwindow.EarliestStartTime, tempwindow.Duration,
					0.5);
			tempwindow.StartTime = BestStartTime; // set the start time at 0.5

			// add this accesswindow to list

			if (tempwindow.Duration >= MinimalObservationDuration) // Omit those
																	// short
																	// windows
				InputAccessWindow.add(tempwindow);

		}

		NumberOfAccessWindow = InputAccessWindow.size();
		NumberOfSatellite = Satellites.size();
		NumberOfInputTarget = Targets.size();
		System.out.println("Total satellites:"+NumberOfSatellite+", Total targets: " + NumberOfInputTarget + ", total profit: " + TotalProfitInput);

		myWorkBook.close();
//		for(AccessWindow a :InputAccessWindow)
//		{
//			LocalDateTime t=a.EarliestStartTime;
//			System.out.print(t+"\n");
//		}
//		
//		LocalDateTime t1=InputAccessWindow.get(0).EarliestStartTime;
//		LocalDateTime t2=InputAccessWindow.get(1).EarliestStartTime;
//		
//		int Compare=t2.compareTo(t1);
//		int Compare2=t1.compareTo(t2);
//		int Compare3=t1.compareTo(t1);
//	

		Collections.sort(InputAccessWindow); // sort according to start time

		AccessWindowList = InputAccessWindow;
		EarliestStartTime = AccessWindowList.get(0).EarliestStartTime;
		LatestEndTime = AccessWindowList.get(NumberOfAccessWindow - 1).LatestStopTime;
		TotalDuration = Duration.between(EarliestStartTime, LatestEndTime);
		GroupByTarget = new HashMap<String, ArrayList<AccessWindow>>();
		// calculate earliest chance for each target
		for (AccessWindow w : AccessWindowList) {
			if (GroupByTarget.containsKey(w.TargetID)) {
				ArrayList<AccessWindow> windowListForThisTarget = GroupByTarget.get(w.TargetID);
				windowListForThisTarget.add(w);
				GroupByTarget.put(w.TargetID, windowListForThisTarget);
			} else {
				ArrayList<AccessWindow> windowListForThisTarget = new ArrayList<AccessWindow>();
				windowListForThisTarget.add(w);

				GroupByTarget.put(w.TargetID, windowListForThisTarget);
			}
		}
		
		//output all start time
//		for(AccessWindow a :AccessWindowList)
//		{
//			LocalDateTime t=a.EarliestStartTime;
//			System.out.print(t+"\n");
//		}
		

	}

	// Two functions to transform start time from double to LocalDateTime, and
	// vice versa
	public static LocalDateTime StartTimeTransformation(LocalDateTime Earliest, double duration,
			double Doublestarttime) {
		LocalDateTime RealstartTime;
		double seconds = duration * Doublestarttime;
		long needMoveSecond = (long) Math.floor(seconds);
		int needMoveNaco = (int) ((seconds - needMoveSecond) * NANOTOSEC);
		RealstartTime = Earliest.plusSeconds(needMoveSecond).plusNanos(needMoveNaco);
		return RealstartTime;
	}

	public static double StartTimeTransformation(LocalDateTime Earliest, double duration, LocalDateTime Realstarttime) {
		double DoublestartTime;
		Duration d = Duration.between(Earliest, Realstarttime);
		long second = d.getSeconds();
		int naco = d.getNano();
		double totalsed = second + naco / NANOTOSEC;
		DoublestartTime = totalsed / duration;
		return DoublestartTime;
	}

	// Quality indicator calculation
	public static double QualityCalculation(double DoubleStartTime) {
		double Q = 0;
		if (DoubleStartTime >= 0 && DoubleStartTime <= 0.5) {
			Q = 1.8 * DoubleStartTime + 0.1;
		} else {
			Q = 2 - 2 * DoubleStartTime;
		}

		return Q;
	}

	// Find the first executed access window from the index backwards, if
	// couldn't find, return the index itself
	public static WindowAndIndex FindTheNextWindow(ArrayList<AccessWindow> windowlist, int index) {
		WindowAndIndex temp = new WindowAndIndex();

		if (index == windowlist.size() - 1) {
			temp.index = index;
			temp.window = windowlist.get(index);
		} else if (index >= -1 && index < windowlist.size() - 1) {
			for (int i = index + 1; i < windowlist.size(); i++)

			{
				if (windowlist.get(i).execute) {
					temp.index = i;
					temp.window = windowlist.get(i);
					break;
				}
				if (i == windowlist.size() - 1)// couldn't find
				{
					temp.index = index;
					temp.window = windowlist.get(index);
				}
			}
		} else {

			System.out.println(
					"Invalid index For FindTheNextWindow, size of window: " + windowlist.size() + ", index : " + index);
		}
		return temp;
	}

	// Find the first executed access window from the index onwards,if couldn't
	// find, return the index itself
	public static WindowAndIndex FindThePreviousWindow(ArrayList<AccessWindow> windowlist, int index) {
		WindowAndIndex temp = new WindowAndIndex();
		if (index == 0) {
			temp.index = 0;
			temp.window = windowlist.get(0);
		} else if (index > 0 && index <= windowlist.size()) {

			// AccessWindow a =new AccessWindow();
			for (int i = index - 1; i >= 0; i--) {
				if (windowlist.get(i).execute) {
					temp.index = i;
					temp.window = windowlist.get(i);
					break;
				}
				if (i == 0) {
					temp.index = index;
					temp.window = windowlist.get(index);
				}
			}

		}

		else {
			System.out.println("Invalid index For FindTheLastWindow,  size of window: " + windowlist.size()
					+ ", index : " + index);

		}
		return temp;

	}

	/* (non-Javadoc)
	 * @see org.moeaframework.core.Problem#evaluate(org.moeaframework.core.Solution)
	 */
	/* (non-Javadoc)
	 * @see org.moeaframework.core.Problem#evaluate(org.moeaframework.core.Solution)
	 */
	@Override
	public void evaluate(Solution solution) {
		// 1.get variable from the solution, give it to AccessWindow, starttime
		// set as 0.5
	boolean[] d = EncodingUtils.getBinary(solution.getVariable(0));
	double[] starttime=EncodingUtils.getDoubleArray(solution.getVariable(1));
	Random rand = new Random(); 

		for (int i = 0; i < d.length; i++) {
			AccessWindow window = AccessWindowList.get(i);

			window.execute = d[i]; // execute variable
			//set the start time as in the gene
			LocalDateTime BestStartTime = StartTimeTransformation(window.EarliestStartTime, window.Duration, starttime[i]);

		//	LocalDateTime BestStartTime = StartTimeTransformation(window.EarliestStartTime, window.Duration, 0.5);
			window.StartTime = BestStartTime; 
			// if(window.execute)
			// ExecutedAccessWindowList.add(window);

		}

		// 2. deal with constraints, change start time first, then bitstring

		// 2.0 group by satellite and target

		// Get all executed windows
		ArrayList<AccessWindow> ToBeDoneAccessWindowList = FindAllExecuteWindows(AccessWindowList);


		
		Map<String, ArrayList<AccessWindow>> AccessWindowGroupByTarget = new HashMap<String, ArrayList<AccessWindow>>();// execute is true
		for (AccessWindow tempWindow : ToBeDoneAccessWindowList) {
			if (AccessWindowGroupByTarget.containsKey(tempWindow.TargetID)) {
				ArrayList<AccessWindow> WindowList = AccessWindowGroupByTarget.get(tempWindow.TargetID);
				WindowList.add(tempWindow);
			} else {
				ArrayList<AccessWindow> WindowList = new ArrayList<AccessWindow>();
				WindowList.add(tempWindow);
				AccessWindowGroupByTarget.put(tempWindow.TargetID, WindowList);
			}

		}
		//2.1 make sure each target is only observed once.
		for (String target:AccessWindowGroupByTarget.keySet())
		{
			ArrayList<AccessWindow> WindowListForOneTarget=FindAllExecuteWindows(AccessWindowGroupByTarget.get(target));
			if(WindowListForOneTarget.size()>1)
			{
				//keep one window randomly
				int keepindex=rand.nextInt(WindowListForOneTarget.size());
				// set all the other windows to false
				for(int i=0;i<WindowListForOneTarget.size();i++)
				{
					if(i!=keepindex)
					{
						WindowListForOneTarget.get(i).execute=false;
					}
				}	
				
			}
			
			
		}
		
		int executeNumber=FindAllExecuteWindows(ToBeDoneAccessWindowList).size();
		
		if(executeNumber>NumberOfInputTarget)
		{
			System.out.println("Constraint 1 is violated");
		}
		// 2.2 adjust start time when constraint is violated, different operators by objectives
		Map<String, ArrayList<AccessWindow>> AccessWindowGroupBySatellite = new HashMap<String, ArrayList<AccessWindow>>();// execute is true
		for (AccessWindow tempWindow : ToBeDoneAccessWindowList) {
			if (AccessWindowGroupBySatellite.containsKey(tempWindow.SatelliteID)) {
				ArrayList<AccessWindow> WindowList = AccessWindowGroupBySatellite.get(tempWindow.SatelliteID);
				WindowList.add(tempWindow);
			} else // put a new key-value
			{
				ArrayList<AccessWindow> WindowList = new ArrayList<AccessWindow>();
				WindowList.add(tempWindow);
				AccessWindowGroupBySatellite.put(tempWindow.SatelliteID, WindowList);

			}
		}
	//	int ConstraintOperator = rand.nextInt(4);
		int ConstraintOperator = 1;//0:high profit,1:high quantity,2:random,3:high timeliness
		for (Map.Entry<String, ArrayList<AccessWindow>> entry1 : AccessWindowGroupBySatellite.entrySet()) {
			// String satellite= entry1.getKey();
			ArrayList<AccessWindow> windowList = entry1.getValue();

			while (ConstraintCheck(windowList)) {

				WindowAndIndex lastExecuteWindow = FindThePreviousWindow(windowList, windowList.size());
				int LastIndex = lastExecuteWindow.index;
				// WindowAndIndex
				// SecondlastExecuteWindow=FindTheLastWindow(windowList,LastIndex);
				// int SecondLastIndex=SecondlastExecuteWindow.index;
				
				int l = -1;

				while (l < LastIndex) {
					WindowAndIndex CurrentWindow = FindTheNextWindow(windowList, l);
					AccessWindow current = CurrentWindow.window;
					int currentIndex = CurrentWindow.index;
					WindowAndIndex NextWindow = FindTheNextWindow(windowList, currentIndex);
					AccessWindow next = NextWindow.window;
					int nextIndex = NextWindow.index;
					if (currentIndex == nextIndex)// no more execute window
					{
						break;
					}
					LocalDateTime currentStartCopy = current.StartTime;
					LocalDateTime earliestStartTimeForNext = current.StartTime
							.plusSeconds((long) (MinimalObservationDuration + MinimalObservationInverval));

					if (earliestStartTimeForNext.isAfter(next.StartTime))// conflict
																			// happens
					{
						// Step1: Try current start time in advance
						Duration duration = Duration.between(next.StartTime, earliestStartTimeForNext);
						long moveseconds = duration.getSeconds();
						int movenano = duration.getNano();
						double totalmove = moveseconds + movenano / NANOTOSEC;

						LocalDateTime CurrentEarliest = current.EarliestStartTime;
						// check if it is conflicted with the last one
						WindowAndIndex LastWindow = FindThePreviousWindow(windowList, currentIndex);
						AccessWindow last = LastWindow.window;
						if (LastWindow.index != currentIndex) {
							LocalDateTime lastEnds = last.StartTime
									.plusSeconds((long) (MinimalObservationDuration + MinimalObservationInverval));
							CurrentEarliest = GetLater(lastEnds, CurrentEarliest);
						}

						// Is currentEarliest before current start time?, then
						// currentmove is negative
						// if(CurrentEarliest.isBefore(current.StartTime))
						Duration currentmove = Duration.between(CurrentEarliest, current.StartTime);
						double currenttotalmove = currentmove.getSeconds() + currentmove.getNano() / NANOTOSEC;
						// step2: Try delay next start time
						if (currenttotalmove > totalmove) // can delay start
															// time to get
															// better quality
						{
							current.StartTime = current.StartTime.minusSeconds(moveseconds).minusNanos(movenano);
						} else if (currenttotalmove < totalmove)// next start
																// time should
																// also be
																// changed
						{
							double needmove = totalmove - currenttotalmove;
							long needMoveSecond = (long) Math.floor(needmove);
							int needMoveNaco = (int) ((needmove - needMoveSecond) * NANOTOSEC);
							current.StartTime = CurrentEarliest;
							LocalDateTime NextNeedToStart = next.StartTime.plusNanos(needMoveNaco)
									.plusSeconds(needMoveSecond);
							if (NextNeedToStart.isBefore(next.LatestStopTime
									.minusSeconds((long) (MinimalObservationDuration + MinimalObservationInverval)))) {// can
																														// move
																														// next
								next.StartTime = NextNeedToStart;
							} else {// constraint cannot be resolved, change
									// decision variable, delete one of the two conflicting windows according to the ConstraintOperator
						switch(ConstraintOperator)	{	
						case 0:{// delete the less profit one
								if (current.profit >= next.profit) {// keep
																	// current,
																	// change
																	// next to
																	// false

									next.execute = false;
									current.StartTime = currentStartCopy;
								} else {// keep next, change current
									current.execute = false;
								}
								break;
						}
						case 1:{// delete the target with fewer chances
							int numberOfCurrent=AccessWindowGroupByTarget.get(current.TargetID).size();
							int numberOfNext = AccessWindowGroupByTarget.get(next.TargetID).size();
							if(numberOfCurrent>numberOfNext)
							{//keep next,delete current
								current.execute = false;
							}
							else
							{//keep current, delete next
								next.execute = false;
								current.StartTime = currentStartCopy;
							}
							break;
						}
						case 2:{// randomly delete one
							boolean randnum=rand.nextBoolean();
							if(randnum){
								//keep next,delete current
								current.execute = false;
							}
							else{
								//keep current, delete next
								next.execute = false;
								current.StartTime = currentStartCopy;
							}
							
							break;
						}
						case 3:{//delete the less timeliness factor one
							double currentFactor = CalculateTimelinessFactor(current, current.TargetID);
							double nextFactor = CalculateTimelinessFactor(next, next.TargetID);
							if(currentFactor>=nextFactor){
								//keep current, delete next
								next.execute = false;
								current.StartTime = currentStartCopy;
							}
							else{
								//keep next,delete current
								current.execute = false;
							}
							break;
						}
						}
							}

						} else {// the next window doesn't need move
							current.StartTime = CurrentEarliest;// move current
																// in advance,
																// no more
																// action
						}

					} // finish constraint handling
					l = nextIndex - 1;

				}
			}

		}

		// 3. calculate objectives: (1)profit, (2)quantity of finished
		// targets,(3)quality,(4)Energy,(5)Resource Equilibrium, (6)timeliness

		double[] f = new double[3]; // objectives
		double[] g = new double[NumberOfSatellite]; // constraints

		ArrayList<String> FinishedTasks = new ArrayList<String>();
		int TotalProfit = 0;
		int TotalFinishedTargets = 0;
		ArrayList<AccessWindow> ExecutedAccessWindowList = new ArrayList<AccessWindow>();// those
																							// execute
																							// variables
																							// are
																							// true

		Map<String, Double> resolutionfactor = new HashMap<String, Double>(); // <targetID,QualityIndicator>
		Map<String, Double> Timelinessfactor = new HashMap<String, Double>(); // <targetID,timelinessIndicator>
		// Get all executed windows
		for (AccessWindow tempWindow : AccessWindowList) {
			if (tempWindow.execute)
				ExecutedAccessWindowList.add(tempWindow);
		}

		for (AccessWindow tempWindow : ExecutedAccessWindowList) {
			if (!FinishedTasks.contains(tempWindow.TargetID)) {
				FinishedTasks.add(tempWindow.TargetID);
				TotalProfit += tempWindow.profit;

			}
			// calculate quality factor when this target has been observed
			// already
			if (resolutionfactor.containsKey(tempWindow.TargetID)) {
				double currentvalue = resolutionfactor.get(tempWindow.TargetID);
				double Doublestarttime = StartTimeTransformation(tempWindow.EarliestStartTime, tempWindow.Duration,
						tempWindow.StartTime);
				double qualityIndicator = QualityCalculation(Doublestarttime);

				if (qualityIndicator > currentvalue)
					resolutionfactor.put(tempWindow.TargetID, qualityIndicator);

			}
			// calculate quality factor when this target hasn't been observed
			// before
			else {
				double Doublestarttime = StartTimeTransformation(tempWindow.EarliestStartTime, tempWindow.Duration,
						tempWindow.StartTime);
				double qualityIndicator = QualityCalculation(Doublestarttime);
				resolutionfactor.put(tempWindow.TargetID, qualityIndicator);
			}
			// calculate timeliness factor
			if (Timelinessfactor.containsKey(tempWindow.TargetID)) {
				double currentvalue = resolutionfactor.get(tempWindow.TargetID);

				double newvalue = CalculateTimelinessFactor(tempWindow, tempWindow.TargetID);
				if (newvalue > currentvalue)
					Timelinessfactor.put(tempWindow.TargetID, newvalue);
			} else {
				double Timelinesfactor = CalculateTimelinessFactor(tempWindow, tempWindow.TargetID);
				Timelinessfactor.put(tempWindow.TargetID, Timelinesfactor);
			}

		}
	

		TotalFinishedTargets = FinishedTasks.size();
		double TotalObservingTime = ExecutedAccessWindowList.size() * MinimalObservationDuration;

     	f[0] = -(double)TotalProfit/TotalProfitInput; // Objective 1: Profit -> max
		
//		f[1] = -(double)TotalFinishedTargets/NumberOfInputTarget; // Objective 2: Quantity -> max

		double OverallQuality = 0;
		for (String key : resolutionfactor.keySet()) {
			OverallQuality += resolutionfactor.get(key);
		}
		//
		double quality = 0;
		if (resolutionfactor.size() != 0) {
			quality = -OverallQuality / resolutionfactor.size();
		}

		f[1] = quality;// objective 3:observing quality, overallquality
		// ->max,

//	    f[3]=TotalObservingTime/(NumberOfInputTarget*MinimalObservationDuration); //Objective 4: Energy Cost -> min

		int[] NumbersBySat = new int[NumberOfSatellite];
		int i = 0;
		for (Map.Entry<String, ArrayList<AccessWindow>> entry1 : AccessWindowGroupBySatellite.entrySet()) {
			String satellite = entry1.getKey();
			ArrayList<AccessWindow> windowListForthisSat = entry1.getValue();
			int ObservationNumberOfThisSat = 0;
			for (AccessWindow w : windowListForthisSat) {
				if (w.execute)
					ObservationNumberOfThisSat++;
			}
			NumbersBySat[i] = ObservationNumberOfThisSat;
			i++;
		}

		// calculate variance
		double sum = 0;
		for (int j = 0; j < NumbersBySat.length; j++) {
			sum += NumbersBySat[j];
		}
		double mean = sum / NumbersBySat.length;
		double variance = 0;
		for (int k = 0; k < NumbersBySat.length; k++) {
			variance = variance + Math.pow(NumbersBySat[k] - mean, 2);
		}
		variance = variance / NumbersBySat.length;
		double standardDeviation = Math.sqrt(variance);
//	    f[4]=standardDeviation/10; // objective 5: Resource equilibrium,
		// variance -> min

		// use Gini coefficient instead of standard deviation
		double GINIUP = 0;
		for (int p = 0; p < NumbersBySat.length; p++)
			for (int q = 0; q < NumbersBySat.length; q++) {
				GINIUP += Math.abs(NumbersBySat[p] - NumbersBySat[q]);
			}
		double GINIDOWN = 2 * Math.pow(NumberOfSatellite, 2) * mean;
		double GINICoefficient = GINIUP / GINIDOWN;
		// f[1] = GINICoefficient; // objective 5: Resource equilibrium, for
		// GINI coefficient 0 is best

		double timtlinessFactor = 0;
		for (Map.Entry<String, Double> entryForTimeliness : Timelinessfactor.entrySet()) {
			double timeForThis = entryForTimeliness.getValue();
			timtlinessFactor += timeForThis;
		}
		if (Timelinessfactor.size() > 0)
			timtlinessFactor = -timtlinessFactor / Timelinessfactor.size();
	    f[2]=timtlinessFactor;// objective 6: Timeliness -> max

		// 4. calculate constraints, should be 0, for check
		Map<String, ArrayList<AccessWindow>> ExecutedAccessWindowGroupBySatellite = new HashMap<String, ArrayList<AccessWindow>>();
		for (AccessWindow tempWindow : ExecutedAccessWindowList) {
			if (ExecutedAccessWindowGroupBySatellite.containsKey(tempWindow.SatelliteID)) {
				ArrayList<AccessWindow> ExistedWindowList = ExecutedAccessWindowGroupBySatellite
						.get(tempWindow.SatelliteID);
				ExistedWindowList.add(tempWindow);
			} else // put a new key-value
			{
				ArrayList<AccessWindow> WindowList = new ArrayList<AccessWindow>();
				WindowList.add(tempWindow);
				ExecutedAccessWindowGroupBySatellite.put(tempWindow.SatelliteID, WindowList);

			}

		}

		int satelliteindex = 0;
		for (Map.Entry<String, ArrayList<AccessWindow>> entry2 : ExecutedAccessWindowGroupBySatellite.entrySet()) {
			// String satellite= entry1.getKey();
			ArrayList<AccessWindow> windowList1 = entry2.getValue();
			double constraintOfthisSat = 0;

			for (int m = 0; m < windowList1.size() - 1; m++) {
				AccessWindow current = windowList1.get(m);
				AccessWindow next = windowList1.get(m + 1);
				LocalDateTime earliestStartTimeForNext = current.StartTime
						.plusSeconds((long) (MinimalObservationDuration + MinimalObservationInverval));
				if (earliestStartTimeForNext.isAfter(next.StartTime)) {
					Duration duration = Duration.between(next.StartTime, earliestStartTimeForNext);
					constraintOfthisSat += duration.getSeconds();
				}
			}
			g[satelliteindex] = constraintOfthisSat;
			satelliteindex++;
		}
		// set objectives
		solution.setObjectives(f);
		// set constraints
		solution.setConstraints(g);

		// 5. set the decision variables

		boolean[] decisionVariableNow = new boolean[AccessWindowList.size()];
		double[] startTimeNow = new double[AccessWindowList.size()];
		for (int m = 0; m < AccessWindowList.size(); m++) {
			AccessWindow aw = AccessWindowList.get(m);
			decisionVariableNow[m] = aw.execute;
			double Doublestarttime = StartTimeTransformation(aw.EarliestStartTime, aw.Duration, aw.StartTime);
			startTimeNow[m] = Doublestarttime;
		}

		EncodingUtils.setBinary(solution.getVariable(0), decisionVariableNow);
		solution.setVariable(1, EncodingUtils.newDoubleArray(startTimeNow));

		// check changes or not
		// boolean[] decisionNow =
		// EncodingUtils.getBinary(solution.getVariable(0));
		// double[]
		// starttimeNow=EncodingUtils.getDoubleArray(solution.getVariable(1));

	}

	private ArrayList<AccessWindow> FindAllExecuteWindows(ArrayList<AccessWindow> accessWindowList2) {
		ArrayList<AccessWindow> results =new ArrayList<AccessWindow>();
		for(AccessWindow a:accessWindowList2){
			if(a.execute)
				results.add(a);
		}
		
		return results;
	}

	private double CalculateTimelinessFactor(AccessWindow tempWindow, String targetID) {
		// TODO get all the access windows from GroupByTarget, then evaluate
		// this window
		double factor = 0;
		ArrayList<AccessWindow> windowsForThisTarget = GroupByTarget.get(targetID);
		if (windowsForThisTarget.size() == 1) {
			factor = 1;
		} else if (windowsForThisTarget.size() > 1) {
			int biggestindex = windowsForThisTarget.size() - 1;
			int thisindex = windowsForThisTarget.indexOf(tempWindow);
			factor = thisindex / (double) (-biggestindex) + 1;

		} else {
			System.out.println("No access window found for this target: " + targetID + " CalculateTimelinessFactor");
		}

		return factor;
	}

	public boolean ConstraintCheck(ArrayList<AccessWindow> windowList) {
		// check whether the constraint is violated , windowList is the list of
		// one satellite
		// TODO Auto-generated method stub
		boolean violate = false;
		ArrayList<AccessWindow> ExecutedWindowForThisSatellite = new ArrayList<AccessWindow>();

		for (AccessWindow a : windowList) {
			if (a.execute)
				ExecutedWindowForThisSatellite.add(a);
		}
		double constraintOfthisSat = 0;
		int ExecuteNum = ExecutedWindowForThisSatellite.size();
		if (ExecuteNum >= 2) {
			for (int m = 0; m < ExecutedWindowForThisSatellite.size() - 1; m++) {
				AccessWindow current = ExecutedWindowForThisSatellite.get(m);
				AccessWindow next = ExecutedWindowForThisSatellite.get(m + 1);
				LocalDateTime earliestStartTimeForNext = current.StartTime
						.plusSeconds((long) (MinimalObservationDuration + MinimalObservationInverval));
				if (earliestStartTimeForNext.isAfter(next.StartTime)) {
					// Duration duration = Duration.between(next.StartTime,
					// earliestStartTimeForNext);
					// constraintOfthisSat +=
					// duration.getSeconds()+duration.getNano()/NANOTOSEC;
					violate = true;
					break;
				}
			}
		}

		return violate;
	}

	public static int GetExecuteNum(ArrayList<AccessWindow> windowList) {
		int NumOfExecute = 0;
		for (int k = 0; k < windowList.size(); k++) {
			if (windowList.get(k).execute)
				NumOfExecute++;
		}

		return NumOfExecute;
	}

	// use after the algorithm ends
	public static void evaluateSolution(Solution solution) {
		// 1.get variable from the solution, give it to AccessWindow, starttime
		// set as 0.5
		boolean[] d = EncodingUtils.getBinary(solution.getVariable(0));
		double[] starttime = EncodingUtils.getDoubleArray(solution.getVariable(1));

		for (int i = 0; i < d.length; i++) {
			AccessWindow window = AccessWindowList.get(i);

			window.execute = d[i]; // execute variable
			LocalDateTime st = StartTimeTransformation(window.EarliestStartTime, window.Duration, starttime[i]);

			window.StartTime = st; // set the start time at 0.5
			// if(window.execute)
			// ExecutedAccessWindowList.add(window);

		}

		//group by target
		GroupByTarget = new HashMap<String, ArrayList<AccessWindow>>();
		// calculate earliest chance for each target
		for (AccessWindow w : AccessWindowList) {
			if(!w.execute)
				continue;
			if (GroupByTarget.containsKey(w.TargetID)) {
				ArrayList<AccessWindow> windowListForThisTarget = GroupByTarget.get(w.TargetID);
				windowListForThisTarget.add(w);
				GroupByTarget.put(w.TargetID, windowListForThisTarget);
			} else {
				ArrayList<AccessWindow> windowListForThisTarget = new ArrayList<AccessWindow>();
				windowListForThisTarget.add(w);

				GroupByTarget.put(w.TargetID, windowListForThisTarget);
			}
		}
		
		for(String target:GroupByTarget.keySet())
		{
			System.out.println(target+": ");
			int num=GroupByTarget.get(target).size();
			System.out.println(num+"\n");
		}
		// 2. deal with constraints, change start time first, then bitstring

		// 2.1 group by satellite
		Map<String, ArrayList<AccessWindow>> AccessWindowGroupBySatellite = new HashMap<String, ArrayList<AccessWindow>>();
		for (AccessWindow tempWindow : AccessWindowList) {
			if (AccessWindowGroupBySatellite.containsKey(tempWindow.SatelliteID)) {
				ArrayList<AccessWindow> WindowList = AccessWindowGroupBySatellite.get(tempWindow.SatelliteID);
				WindowList.add(tempWindow);
			} else // put a new key-value
			{
				ArrayList<AccessWindow> WindowList = new ArrayList<AccessWindow>();
				WindowList.add(tempWindow);
				AccessWindowGroupBySatellite.put(tempWindow.SatelliteID, WindowList);

			}

		}
		
		

		
		// 2.2 adjust start time when constraint is violated

		// for (Map.Entry<String, ArrayList<AccessWindow>> entry1 :
		// AccessWindowGroupBySatellite.entrySet()) {
		// // String satellite= entry1.getKey();
		// ArrayList<AccessWindow> windowList = entry1.getValue();
		// WindowAndIndex lastExecuteWindow = FindTheLastWindow(windowList,
		// windowList.size());
		// int LastIndex = lastExecuteWindow.index;
		// WindowAndIndex SecondlastExecuteWindow =
		// FindTheLastWindow(windowList, LastIndex);
		// int SecondLastIndex = SecondlastExecuteWindow.index;
		// if ((LastIndex != SecondLastIndex) && (LastIndex !=
		// windowList.size()))// make
		// // sure
		// // there
		// // are
		// // at
		// // least
		// // two
		// // execute
		// // windows
		// {
		// int l = -1;
		//
		// while (l < SecondLastIndex) {
		// WindowAndIndex CurrentWindow = FindTheNextWindow(windowList, l);
		// AccessWindow current = CurrentWindow.window;
		// int currentIndex = CurrentWindow.index;
		// WindowAndIndex NextWindow = FindTheNextWindow(windowList,
		// currentIndex);
		// AccessWindow next = NextWindow.window;
		// int nextIndex = NextWindow.index;
		//
		// LocalDateTime currentStartCopy = current.StartTime;
		// LocalDateTime earliestStartTimeForNext = current.StartTime
		// .plusSeconds((long) (MinimalObservationDuration +
		// MinimalObservationInverval));
		//
		// if (earliestStartTimeForNext.isAfter(next.StartTime))// conflict
		// // happens
		// {
		// // Step1: Try current start time in advance
		// Duration duration = Duration.between(next.StartTime,
		// earliestStartTimeForNext);
		// long moveseconds = duration.getSeconds();
		// int movenano = duration.getNano();
		// double totalmove = moveseconds + movenano / NANOTOSEC;
		//
		// LocalDateTime CurrentEarliest = current.EarliestStartTime;
		// // check if it is conflicted with the last one
		// WindowAndIndex LastWindow = FindTheLastWindow(windowList,
		// currentIndex);
		// AccessWindow last = LastWindow.window;
		// if (LastWindow.index != currentIndex) {
		// LocalDateTime lastEnds = last.StartTime
		// .plusSeconds((long) (MinimalObservationDuration +
		// MinimalObservationInverval));
		// CurrentEarliest = GetLater(lastEnds, CurrentEarliest);
		// }
		//
		// // Is currentEarliest before current start time?, then
		// // currentmove is negative
		// // if(CurrentEarliest.isBefore(current.StartTime))
		// Duration currentmove = Duration.between(CurrentEarliest,
		// current.StartTime);
		// double currenttotalmove = currentmove.getSeconds() +
		// currentmove.getNano() / NANOTOSEC;
		// // step2: Try delay next start time
		// if (currenttotalmove > totalmove) // can delay start
		// // time to get
		// // better quality
		// {
		// current.StartTime =
		// current.StartTime.minusSeconds(moveseconds).minusNanos(movenano);
		// } else if (currenttotalmove < totalmove)// next start
		// // time should
		// // also be
		// // changed
		// {
		// double needmove = totalmove - currenttotalmove;
		// long needMoveSecond = (long) Math.floor(needmove);
		// int needMoveNaco = (int) ((needmove - needMoveSecond) * NANOTOSEC);
		// current.StartTime = CurrentEarliest;
		// LocalDateTime NextNeedToStart =
		// next.StartTime.plusNanos(needMoveNaco)
		// .plusSeconds(needMoveSecond);
		// if (NextNeedToStart.isBefore(next.LatestStopTime
		// .minusSeconds((long) (MinimalObservationDuration +
		// MinimalObservationInverval)))) {// can
		// // move
		// // next
		// next.StartTime = NextNeedToStart;
		// } else {// constraint cannot be resolved, change
		// // decision variable
		// if (current.profit >= next.profit) {// keep
		// // current,
		// // change
		// // next to
		// // false
		//
		// next.execute = false;
		// current.StartTime = currentStartCopy;
		// } else {// keep next, change current
		// current.execute = false;
		// }
		//
		// }
		//
		// } else {
		// current.StartTime = CurrentEarliest;// move current
		// // in advance,
		// // no more
		// // action
		// }
		//
		// } // finish constraint handling
		// l = nextIndex - 1;
		//
		// }
		// }
		// }

		// 3. calculate objectives: (1)profit, (2)quantity of finished
		// targets,(3)quality,(4)Energy,(5)Resource Equilibrium

//		double[] f = new double[3]; // objectives
//		double[] g = new double[NumberOfSatellite]; // constraints
//
//		ArrayList<String> FinishedTasks = new ArrayList<String>();
//		int TotalProfit = 0;
//		int TotalFinishedTargets = 0;
//		ArrayList<AccessWindow> ExecutedAccessWindowList = new ArrayList<AccessWindow>();// those
//																							// execute
//																							// variables
//																							// are
//																							// true
//
//		Map<String, Double> resolutionfactor = new HashMap<String, Double>(); // <targetID,QualityIndicator>
//		// Get all executed windows
//		for (AccessWindow tempWindow : AccessWindowList) {
//			if (tempWindow.execute)
//				ExecutedAccessWindowList.add(tempWindow);
//		}
//
//		for (AccessWindow tempWindow : ExecutedAccessWindowList) {
//			if (!FinishedTasks.contains(tempWindow.TargetID)) {
//				FinishedTasks.add(tempWindow.TargetID);
//				TotalProfit += tempWindow.profit;
//
//			}
//			// calculate quality factor when this target has been observed
//			// already
//			if (resolutionfactor.containsKey(tempWindow.TargetID)) {
//				double currentvalue = resolutionfactor.get(tempWindow.TargetID);
//				double Doublestarttime = StartTimeTransformation(tempWindow.EarliestStartTime, tempWindow.Duration,
//						tempWindow.StartTime);
//				double qualityIndicator = QualityCalculation(Doublestarttime);
//
//				if (qualityIndicator > currentvalue)
//					resolutionfactor.put(tempWindow.TargetID, qualityIndicator);
//
//			}
//			// calculate quality factor when this target hasn't been observed
//			// before
//			else {
//				double Doublestarttime = StartTimeTransformation(tempWindow.EarliestStartTime, tempWindow.Duration,
//						tempWindow.StartTime);
//				double qualityIndicator = QualityCalculation(Doublestarttime);
//				resolutionfactor.put(tempWindow.TargetID, qualityIndicator);
//			}
//
//		}
//		TotalFinishedTargets = FinishedTasks.size();
//		double TotalObservingTime = ExecutedAccessWindowList.size() * MinimalObservationDuration;
//
//		f[0] = -TotalProfit; // Objective 1: Profit -> max
//		f[1] = -TotalFinishedTargets; // Objective 2: Quantity -> max
//
//		double OverallQuality = 0;
//		for (String key : resolutionfactor.keySet()) {
//			OverallQuality += resolutionfactor.get(key);
//		}
//		f[2] = -OverallQuality / resolutionfactor.size(); // objective 3:
//		// observing quality, overallquality ->max, 1 is the best
//
//		// f[1]=TotalObservingTime; //Objective 4: Energy Cost -> min
//
//		int[] NumbersBySat = new int[NumberOfSatellite];
//		int i = 0;
//		for (Map.Entry<String, ArrayList<AccessWindow>> entry1 : AccessWindowGroupBySatellite.entrySet()) {
//			String satellite = entry1.getKey();
//			ArrayList<AccessWindow> windowListForthisSat = entry1.getValue();
//			int ObservationNumberOfThisSat = 0;
//			for (AccessWindow w : windowListForthisSat) {
//				if (w.execute)
//					ObservationNumberOfThisSat++;
//			}
//			NumbersBySat[i] = ObservationNumberOfThisSat;
//			i++;
//		}
//
//		// calculate variance
//		double sum = 0;
//		for (int j = 0; j < NumbersBySat.length; j++) {
//			sum += NumbersBySat[j];
//		}
//		double mean = sum / NumbersBySat.length;
//		double variance = 0;
//		for (int k = 0; k < NumbersBySat.length; k++) {
//			variance = variance + Math.pow(NumbersBySat[k] - mean, 2);
//		}
//		variance = variance / NumbersBySat.length;
//		double standardDeviation = Math.sqrt(variance);
//		// f[1]=standardDeviation; // objective 5: Resource equilibrium,
//		// variance -> min
//
//		// use Gini coefficient instead of standard deviation
//		double GINIUP = 0;
//		for (int p = 0; p < NumbersBySat.length; p++)
//			for (int q = 0; q < NumbersBySat.length; q++) {
//				GINIUP += Math.abs(NumbersBySat[p] - NumbersBySat[q]);
//			}
//		double GINIDOWN = 2 * Math.pow(NumberOfSatellite, 2) * mean;
//		double GINICoefficient = GINIUP / GINIDOWN;
//		// f[1] = GINICoefficient; // objective 5: Resource equilibrium, GINI
//		// coefficient -> min
//		// 5. calculate constraints, should be 0, for check
//		Map<String, ArrayList<AccessWindow>> ExecutedAccessWindowGroupBySatellite = new HashMap<String, ArrayList<AccessWindow>>();
//		for (AccessWindow tempWindow : ExecutedAccessWindowList) {
//			if (ExecutedAccessWindowGroupBySatellite.containsKey(tempWindow.SatelliteID)) {
//				ArrayList<AccessWindow> ExistedWindowList = ExecutedAccessWindowGroupBySatellite
//						.get(tempWindow.SatelliteID);
//				ExistedWindowList.add(tempWindow);
//			} else // put a new key-value
//			{
//				ArrayList<AccessWindow> WindowList = new ArrayList<AccessWindow>();
//				WindowList.add(tempWindow);
//				ExecutedAccessWindowGroupBySatellite.put(tempWindow.SatelliteID, WindowList);
//
//			}
//
//		}
//
//		int satelliteindex = 0;
//		for (Map.Entry<String, ArrayList<AccessWindow>> entry2 : ExecutedAccessWindowGroupBySatellite.entrySet()) {
//			// String satellite= entry1.getKey();
//			ArrayList<AccessWindow> windowList1 = entry2.getValue();
//			double constraintOfthisSat = 0;
//
//			for (int m = 0; m < windowList1.size() - 1; m++) {
//				AccessWindow current = windowList1.get(m);
//				AccessWindow next = windowList1.get(m + 1);
//				LocalDateTime earliestStartTimeForNext = current.StartTime
//						.plusSeconds((long) (MinimalObservationDuration + MinimalObservationInverval));
//				if (earliestStartTimeForNext.isAfter(next.StartTime)) {
//					Duration duration = Duration.between(next.StartTime, earliestStartTimeForNext);
//					constraintOfthisSat += duration.getSeconds();
//				}
//			}
//			g[satelliteindex] = constraintOfthisSat;
//			satelliteindex++;
//		}
//		// set objectives
//		solution.setObjectives(f);
//		// set constraints
//		solution.setConstraints(g);

		// set the decision variable

		// boolean[] decisionVariableNow = new boolean[AccessWindowList.size()];
		// double[] startTimeNow = new double[AccessWindowList.size()];
		// for (int m = 0; m < AccessWindowList.size(); m++) {
		// AccessWindow aw = AccessWindowList.get(m);
		// decisionVariableNow[m] = aw.execute;
		// double Doublestarttime =
		// StartTimeTransformation(aw.EarliestStartTime, aw.Duration,
		// aw.StartTime);
		// startTimeNow[m] = Doublestarttime;
		// }

		// EncodingUtils.setBinary(solution.getVariable(0),
		// decisionVariableNow);
		// solution.setVariable(1, EncodingUtils.newDoubleArray(startTimeNow));

	}

	@Override
	public String getName() {
		return "SatelliteMissionPlanning";
	}

	@Override
	public int getNumberOfConstraints() {
		return NumberOfSatellite;
	}

	@Override
	public int getNumberOfObjectives() {
		return 3;
	}

	@Override
	public int getNumberOfVariables() {
		return 2;
		// return 1;
	}

	@Override
	public void close() {
		// do nothing
	}

	@Override
	public Solution newSolution() {
		Solution solution = new Solution(2,3, NumberOfSatellite);
		solution.setVariable(0, EncodingUtils.newBinary(NumberOfAccessWindow));
		solution.setVariable(1, EncodingUtils.newDoubleArray(NumberOfAccessWindow));
		return solution;
	}

	public void ReadInputFile() {

	}
}
