/* Copyright 2009-2016 David Hadka
 *
 * This file is part of the MOEA Framework.
 *
 * The MOEA Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * The MOEA Framework is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the MOEA Framework.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.moeaframework.examples.ga.knapsack;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import org.moeaframework.Executor;
import org.moeaframework.analysis.plot.Plot;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.Population;
import org.moeaframework.core.PopulationIO;
import org.moeaframework.core.Solution;
import org.moeaframework.util.Vector;

/**
 * Example of binary optimization using the {@link Knapsack} problem on the
 * {@code knapsack.100.2} instance.
 */
public class KnapsackExample {

	/**
	 * Starts the example running the knapsack problem.
	 * 
	 * @param args the command line arguments
	 * @throws IOException if an I/O error occurred
	 */
	public static void main(String[] args) throws IOException {
		// open the file containing the knapsack problem instance
		InputStream input = Knapsack.class.getResourceAsStream(
				"knapsack.100.3");
		
		if (input == null) {
			System.err.println("Unable to find the file knapsack.100.2");
			System.exit(-1);
		}
				
		// solve using NSGA-II
		NondominatedPopulation result = new Executor()
				.withProblemClass(Knapsack.class, input)
				.withAlgorithm("T-NSGA-II")
				.withMaxEvaluations(100000)
				.distributeOnAllCores()
				.run();

		Population Rpopulation = new Population();
		int numberOfObj = result.get(0).getNumberOfObjectives();
		
		// print the results
		for (int i = 0; i < result.size(); i++) {
			Solution solution = result.get(i);
			double[] objectives = solution.getObjectives();
					
			// negate objectives to return them to their maximized form
			objectives = Vector.negate(objectives);
			
			//added by Yali
			
			Solution Rsolution = result.get(i);
			
			for (int j = 0;j<numberOfObj;j++)
			{
				Rsolution.setObjective(j, -Rsolution.getObjective(j));
			}
			Rpopulation.add(Rsolution);
			
			//end Yali
					
			System.out.println("Solution " + (i+1) + ":");
			System.out.println("    Sack 1 Profit: " + objectives[0]);
			System.out.println("    Sack 2 Profit: " + objectives[1]);
			System.out.println("    Sack 3 Profit: " + objectives[2]);
			System.out.println("    Binary String: " + solution.getVariable(0));
		}
		
		
		new Plot()
		.add("NSGAII", Rpopulation)
		.show();

		PopulationIO.writeObjectives(new File("output.txt"), Rpopulation);
	}

}
