package org.moeaframework.algorithm;

import java.io.Serializable;
import java.util.Comparator;

import org.moeaframework.Executor;
import org.moeaframework.core.FastNondominatedSorting;
import org.moeaframework.core.FitnessEvaluator;
import org.moeaframework.core.Initialization;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.NondominatedSorting;
import org.moeaframework.core.Population;
import org.moeaframework.core.Problem;
import org.moeaframework.core.Selection;
import org.moeaframework.core.Solution;
import org.moeaframework.core.Variation;
import org.moeaframework.core.comparator.ChainedComparator;
import org.moeaframework.core.comparator.FitnessComparator;
import org.moeaframework.core.comparator.GenerationalDistanceComparator;
import org.moeaframework.core.comparator.NondominatedSortingComparator;
import org.moeaframework.core.comparator.RankComparator;
import org.moeaframework.core.fitness.AHDFitnessEvaluator;
import org.moeaframework.core.operator.TournamentSelection;

public class ASEMOA extends AbstractEvolutionaryAlgorithm {

	/**
	 * The fitness evaluator to use (e.g., hypervolume or additive-epsilon
	 * indicator).
	 */
	private FitnessEvaluator fitnessEvaluator;
	
	/**
	 * The selection operator.
	 */
	private Selection selection;
	
	/**
	 * The variation operator.
	 */
	private Variation variation;
	
	public static final String FITNESS_ATTRIBUTE = "fitness";
	// GD1
	public static final String GD_ATTRIBUTE = "GD";
	
	public ASEMOA(Problem problem, Initialization initialization, Variation variation2,
			FitnessEvaluator fitnessEvaluator2) {
		super(problem,
				new Population(),
				null,
				initialization);
		this.variation = variation2;
		this.fitnessEvaluator = fitnessEvaluator2;
		
	
		selection = new TournamentSelection(
					new ASEMOAFitnessComparator1());
	
	}

	@Override
	protected void initialize() {
		super.initialize();
		
		// rank the solutions, no nondominated ranking is needed, only want to find the nondominated set
//		new FastNondominatedSorting().evaluate(population);
//		new NondominatedSorting().evaluate(population);
		NondominatedPopulation np=new NondominatedPopulation();
		np.addAll(population);
		population.clear();
		population.addAll(np);
		fitnessEvaluator.evaluate(population);
		
		
	}
	
	@Override
	protected void iterate() {
		int AspirationSetNumber=AHDFitnessEvaluator.NumberOfAspirationPoint;
		int populationSize = population.size();
		// select from dominated set
	
//      NondominatedPopulation=GetNondominatedPopulation(population);
		Solution[] parents = selection.select(variation.getArity(), population);
		Solution[] offspring = variation.evolve(parents);
		
		evaluate(offspring[0]);
		offspring[0].setAttribute(FITNESS_ATTRIBUTE, 0.0);
		offspring[0].setAttribute(GD_ATTRIBUTE, 0.0);
		population.add(offspring[0]);
	
		// find nondominated set
		NondominatedPopulation np=new NondominatedPopulation();
		np.addAll(population);
		
		population.clear();
		population.addAll(np);
		fitnessEvaluator.evaluate(population);
		
//		Population NondominatedPopulation1=GetNondominatedPopulation(population);
		
	
		if(population.size()>AspirationSetNumber)
	
		{
//		Solution removeSolution=FindTheSolutionToBeRemoved(NondominatedPopulation1);
				
			population.sort( new ASEMOAFitnessComparator1());
			
			//remove the worst one
		    population.remove(population.size()-1);
		}
		
	}



	private Population GetNondominatedPopulation(Population population) {
		Population p = new Population();
		for(Solution s: population){
			int rank=(int) s.getAttribute("rank");
			if(rank==0){
				p.add(s);
			}
		}	
		return p;
	}

	/**
	 * Computes the fitness for the last front.
	 */
	private void computeFitnessForLastFront() {
		Population front = new Population();
		int rank = 0;
		
		for (Solution solution : population) {
			int solutionRank = (Integer)solution.getAttribute(
					FastNondominatedSorting.RANK_ATTRIBUTE);
			
			if (solutionRank > rank) {
				front.clear();
				rank = solutionRank;
			}
			
			if (solutionRank == rank) {
				front.add(solution);
			}
			
			solution.setAttribute(FitnessEvaluator.FITNESS_ATTRIBUTE, 0.0);
			solution.setAttribute(FitnessEvaluator.GD_ATTRIBUTE, 0.0);
		}
		
		fitnessEvaluator.evaluate(front);
	}
	
	
//	public class ASEMOAFitnessComparator extends ChainedComparator
//	implements Comparator<Solution>, Serializable {
//
//		private static final long serialVersionUID = -4088873047790962685L;
//
//		public ASEMOAFitnessComparator() {
//			//super(new RankComparator(), new FitnessComparator(fitnessEvaluator.areLargerValuesPreferred()));
//
//			super(new RankComparator(), new FitnessComparator(fitnessEvaluator.areLargerValuesPreferred()),new GenerationalDistanceComparator());
//		}
//
//	}
	
	public class ASEMOAFitnessComparator1 extends ChainedComparator
	implements Comparator<Solution>, Serializable {

		private static final long serialVersionUID = -4088873047790962685L;

		public ASEMOAFitnessComparator1() {
			//super(new RankComparator(), new FitnessComparator(fitnessEvaluator.areLargerValuesPreferred()));

			super( new FitnessComparator(fitnessEvaluator.areLargerValuesPreferred()),new GenerationalDistanceComparator());
		}

	}


	@Override
	public Population getCurrentPopulation() {
		// TODO Auto-generated method stub
		return null;
	}
	

	
}
