/* Copyright 2009-2016 David Hadka
 *
 * This file is part of the MOEA Framework.
 *
 * The MOEA Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * The MOEA Framework is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the MOEA Framework.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.moeaframework.analysis.collector;

import java.util.ArrayList;
import java.util.List;

import org.moeaframework.core.EvolutionaryAlgorithm;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.Population;
import org.moeaframework.core.Solution;

/**
 * Collects the population size and archive size, if available, from an
 * {@link EvolutionaryAlgorithm}.
 */
public class PopulationSizeInROI implements Collector {
	public static double[][] Preferences; //ZDT1
	public static int NumOfROI;
	public static String Shape;
	/**
	 * The algorithm instance used by this collector; or {@code null} if this 
	 * collector has not yet been attached.
	 */
	private final EvolutionaryAlgorithm algorithm;
	
	/**
	 * Constructs an unattached collector for recording the population size and
	 * archive size, if available, from an {@code EvolutionaryAlgorithm}.
	 * @param preference 
	 * @param numOfROI2 
	 */
	public PopulationSizeInROI(int numOfROI2, double[][] preference,String shape) {
		this(null);
		this.NumOfROI=numOfROI2;
		this.Preferences=preference;
		this.Shape=shape;
		
	}
	
	/**
	 * Constructs a collector for recording the population size and archive
	 * size, if available, from the specified {@code EvolutionaryAlgorithm}.
	 * 
	 * @param algorithm the algorithm this collector records data from
	 */
	public PopulationSizeInROI(EvolutionaryAlgorithm algorithm) {
		super();
		this.algorithm = algorithm;
	}

	@Override
	public AttachPoint getAttachPoint() {
		return AttachPoint.isSubclass(EvolutionaryAlgorithm.class).and(
				AttachPoint.not(AttachPoint.isNestedIn(
						EvolutionaryAlgorithm.class)));
	}

	@Override
	public Collector attach(Object object) {
		return new PopulationSizeInROI(
				(EvolutionaryAlgorithm)object);
	}

	@Override
	public void collect(Accumulator accumulator) {

		Population p=algorithm.getPopulation();
		NondominatedPopulation np=new NondominatedPopulation();
		np.addAll(p);

		int NumOfObj=p.get(0).getNumberOfObjectives();
		int NumResult=0;
		for(Solution s:p){
			boolean InROI=false;
			for(int i=0;i<NumOfROI;i++){
				//check this ROI
				if(Shape=="rectangle"){
				double[] min=new double[NumOfObj];
				double[] max=new double[NumOfObj];
				for(int j=0;j<NumOfObj;j++){
					min[j]=Preferences[i][j];
					max[j]=Preferences[i][j+NumOfObj];
				}
				boolean  inThisRegion=CheckInROI(s,min,max); 
				if(inThisRegion)
				{
					InROI=true;
				}
				}
				else if(Shape=="circle")
				{
					double[] center=new double[NumOfObj];				
					for(int j=0;j<NumOfObj;j++){
						center[j]=Preferences[i][j];	
					}
					double radis=Preferences[i][NumOfObj];
					boolean  inThisRegion=CheckInROICircle(s,center,radis); 
									
					if(inThisRegion)
					{
						InROI=true;
					}
				}
			}
			if(InROI)
			{
				NumResult++;
			}	
		
		}
			
		accumulator.add("Population Size In ROI", NumResult);
		
		
	}
	
	private boolean CheckInROICircle(Solution s, double[] center, double radis) {
		boolean InROI=false;
		int ObjNum=s.getNumberOfObjectives();
		double distance=0;
		for (int i=0;i<ObjNum;i++)
		{
			distance+=Math.pow(Math.abs(s.getObjective(i)- center[i]),2);
		
		}
		if (distance <= Math.pow(radis, 2))// in the region
		{
			InROI=true;
		}
		
		
		return InROI;
	}
	
	
	public boolean CheckInROI(Solution s, double[] minBound, double[] maxBound){
		boolean InROI=true;
		for(int i=0;i<s.getNumberOfObjectives();i++)
			if((s.getObjective(i)>maxBound[i])||(s.getObjective(i)<minBound[i])){
				InROI=false;
				break;			
		}
		return InROI;
	}
	

}
