package org.moeaframework.core.comparator;

import java.io.Serializable;

import org.moeaframework.core.FastNondominatedSorting;
import org.moeaframework.core.Solution;
//used in CP-NSGA-II
public class ChebycheffPreferenceComparator implements DominanceComparator,
Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 10L;
	public ChebycheffPreferenceComparator(){
		super();
	}

	@Override
	public int compare(Solution solution1, Solution solution2) {
	    int result=0;
	    boolean s1InROI=CheckInROI(solution1);
	    boolean s2InROI=CheckInROI(solution2);
	    
	    if(s1InROI&&s2InROI)// both in ROI, according to normal Pareto dominance relation
	    {
	    	ParetoObjectiveComparator p=new ParetoObjectiveComparator();
	    	result=p.compare(solution1, solution2);	    	
	    	
	    }
	    else
	    {
	    	double dis1=(double)solution1.getAttribute("ChebycheffASFValue");
	       	double dis2=(double)solution2.getAttribute("ChebycheffASFValue");
	       	if(dis1<dis2){
	       		result=-1;
	       	}
	       	else if(dis1>dis2){
	       		result=1;
	       	}
	       	else{
	       		result=0;
	       	}
	    	
	    }
	    
		
		
		
		return result;
		
	}

	private boolean CheckInROI(Solution solution1) {
		boolean result=false;
		double tao=0.5;
		double maxASF=FastNondominatedSorting.MaxChebyASF;
		double minASF=FastNondominatedSorting.MinChebyASF;
		double delta=tao*(maxASF-minASF);
		double dis=(double)solution1.getAttribute("ChebycheffASFValue");
		if(dis<=minASF+delta)
		{
			result=true;
		}
		
		
		
		return result;
	}
		
		
}
