package org.moeaframework.core.comparator;
import java.io.Serializable;

import org.moeaframework.core.FastNondominatedSorting;
import org.moeaframework.core.Solution;


//r-dominance relation used in r-NSGA-II

public class RDominanceComparator implements DominanceComparator,
Serializable {
	
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public RDominanceComparator() {
		super();
	}
	@Override
	public int compare(Solution solution1, Solution solution2) {
		boolean dominate1 = false;
		boolean dominate2 = false;
	
		for (int i = 0; i < solution1.getNumberOfObjectives(); i++) {
			if (solution1.getObjective(i) < solution2.getObjective(i)) {
				dominate1 = true;

				if (dominate2) {
					if(RDominanceCheck(solution1,solution2))
					{return -1;}
					else
					{
						return 0;
					}
				}
			} else if (solution1.getObjective(i) > solution2.getObjective(i)) {
				dominate2 = true;

				if (dominate1) {
					if(RDominanceCheck(solution1,solution2))
					{return -1;}
					else
					{
						return 0;
					}
				}
			}
		}

		if (dominate1 == dominate2) {
			if(RDominanceCheck(solution1,solution2))
			{return -1;}
			else
			{
				return 0;
			}
		} else if (dominate1) {
			return -1;
		} else {
			return 1;
		}
				
		}
	// if solution1 r-dominates solution2 in the second condition, return true, otherwise return false
		private boolean RDominanceCheck(Solution solution1, Solution solution2) {
			
			double delta=0.1;//spread control in r-dominance
			double Dmax=FastNondominatedSorting.MaxDistance;
			double Dmin=FastNondominatedSorting.MinDistance;
			
			double dis1=(double)solution1.getAttribute("WeightedEuclideanDistanceToRP");
			double dis2=(double)solution2.getAttribute("WeightedEuclideanDistanceToRP");
			
			double value=(dis1-dis2)/(Dmax-Dmin);
			if(value<-delta)
			{
				return true;
			}
			
			else{
			   return false;}
		}
		
	
}