package org.moeaframework.core.fitness;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.moeaframework.core.FitnessEvaluator;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.Population;
import org.moeaframework.core.Problem;
import org.moeaframework.core.Solution;
import org.moeaframework.core.indicator.IndicatorUtils;

// Averaged Hausdorff distance fitness evaluator, used in AS-EMOA
public class AHDFitnessEvaluator implements FitnessEvaluator{
	public static final String RANK_ATTRIBUTE = "rank";
	private final Problem problem;
	// preference information
	public static int NumOfROI=1;
//	public static double[][] regions={{0.0,0.8, 0.15,1.0},{0.5,0.65,0.6,0.8},{0.85,0.0,1.0,0.2}};//ZDT2
//	public static double[][] regions={{0.6,0.0, 1,0.3}}; //ZDT1
//	public static double[][] regions={{0.6,0.6, 0.8,0.8}}; //ZDT1
//	public static double[][] regions={{0.08,0.2, 0.18,0.4},{0.7,-0.4,0.8,-0.2}};//ZDT3
//	public static double[][] regions={{0.1,0.25, 0.15,0.3},{0.34,-0.55,0.36,-0.5}};//ZDT3
//	public static double[][] regions={{0.7,0.1,0.9,0.3}}; //ZDT4
//	public static double[][] regions={{0.5,0.5,0.5, 0.7,0.7,0.7}};//DTLZ3_3
//	public static double[][] regions={{0.1,0.8,0.2,0.9},{0.8,0.1,0.9,0.2}}; //DTLZ2-ASEMOA
	public static double[][] regions={{0.6,0.6,0.7,0.9}};
	// for normalization
	public static double[] minimumBound;
	public static double[] maximumBound;
	public static int NumberOfAspirationPoint=20;
//	public static int NumberOfAspirationPoint=21;// for 3D problem
	public static boolean GenerateAspirationSets=true;
	public static NondominatedPopulation AspirationSet;
	
	public AHDFitnessEvaluator(Problem problem) {
		super();
		this.problem = problem;
	}

	@Override
	public void evaluate(Population population) {
		if(GenerateAspirationSets){
			
			// Generate aspiration sets from preference regions
			int NumberOfObj=population.get(0).getNumberOfObjectives();
			int NumberOfVar=population.get(0).getNumberOfVariables();
			minimumBound=new double[NumberOfObj];
			maximumBound=new double[NumberOfObj];
			for(int i=0;i<NumberOfObj;i++){
				minimumBound[i]=Double.POSITIVE_INFINITY;
				maximumBound[i]=Double.NEGATIVE_INFINITY;
			}
			List<Solution> AS=new ArrayList<Solution>();			
			if(NumberOfObj==2){
				// 2 objective, aspiration set is the diagonal line of preference region
				for(int r=0;r<NumOfROI;r++){
					// for each preference region
					int NumberForThisRegion=(int) Math.ceil(NumberOfAspirationPoint/NumOfROI);
					double xmin=regions[r][0];
					minimumBound[0]=(xmin<minimumBound[0])?xmin:minimumBound[0];
					double ymin=regions[r][1];
					minimumBound[1]=(ymin<minimumBound[1])?ymin:minimumBound[1];
					double xmax=regions[r][2];
					maximumBound[0]=(xmax>maximumBound[0])?xmax:maximumBound[0];
					double ymax=regions[r][3];
					maximumBound[1]=(ymax>maximumBound[1])?ymax:maximumBound[1];
					
					for(int k=0;k<NumberForThisRegion;k++){
						Solution s =new Solution(NumberOfVar,NumberOfObj);
						s.setObjective(0, xmin+k*(xmax-xmin)/(NumberForThisRegion-1));
						s.setObjective(1, ymax-k*(ymax-ymin)/(NumberForThisRegion-1));
						AS.add(s);
					}				
				}// for this region
				
			}
			else if(NumberOfObj==3)
			{
				// 3 objective, aspiration set is the 8 vertexes, middle points of 12 edges and the central point.
				for(int r=0;r<NumOfROI;r++){
					// for each preference region
					int NumberForThisRegion=(int) Math.ceil(NumberOfAspirationPoint/NumOfROI);
					double xmin=regions[r][0];
					minimumBound[0]=(xmin<minimumBound[0])?xmin:minimumBound[0];
					double ymin=regions[r][1];
					minimumBound[1]=(ymin<minimumBound[1])?ymin:minimumBound[1];
					double zmin=regions[r][2];
					minimumBound[2]=(zmin<minimumBound[2])?zmin:minimumBound[2];
					double xmax=regions[r][3];
					maximumBound[0]=(xmax>maximumBound[0])?xmax:maximumBound[0];
					double ymax=regions[r][4];
					maximumBound[1]=(ymax>maximumBound[1])?ymax:maximumBound[1];
					double zmax=regions[r][5];
					maximumBound[2]=(zmax>maximumBound[2])?zmax:maximumBound[2];
					

					// 8 vertexes.
					Solution s4 =new Solution(NumberOfVar,NumberOfObj);
					s4.setObjective(0, xmin);
					s4.setObjective(1, ymin);
					s4.setObjective(2, zmin);
					AS.add(s4);
					Solution s1 =new Solution(NumberOfVar,NumberOfObj);
					s1.setObjective(0, xmax);
					s1.setObjective(1, ymin);
					s1.setObjective(2, zmin);
					AS.add(s1);
					Solution s2 =new Solution(NumberOfVar,NumberOfObj);
					s2.setObjective(0, xmax);
					s2.setObjective(1, ymax);
					s2.setObjective(2, zmin);
					AS.add(s2);
					Solution s3 =new Solution(NumberOfVar,NumberOfObj);
					s3.setObjective(0, xmin);
					s3.setObjective(1, ymax);
					s3.setObjective(2, zmin);
					AS.add(s3);
					Solution s5 =new Solution(NumberOfVar,NumberOfObj);
					s5.setObjective(0, xmax);
					s5.setObjective(1, ymin);
					s5.setObjective(2, zmax);
					AS.add(s5);
					Solution s6 =new Solution(NumberOfVar,NumberOfObj);
					s6.setObjective(0, xmin);
					s6.setObjective(1, ymin);
					s6.setObjective(2, zmax);
					AS.add(s6);
					Solution s7 =new Solution(NumberOfVar,NumberOfObj);
					s7.setObjective(0, xmin);
					s7.setObjective(1, ymax);
					s7.setObjective(2, zmax);
					AS.add(s7);
					Solution s8 =new Solution(NumberOfVar,NumberOfObj);
					s8.setObjective(0, xmax);
					s8.setObjective(1, ymax);
					s8.setObjective(2, zmax);
					AS.add(s8);
					// central point
					Solution s9 =new Solution(NumberOfVar,NumberOfObj);
					s9.setObjective(0, (xmax+xmin)/2);
					s9.setObjective(1, (ymax+ymin)/2);
					s9.setObjective(2, (zmax+zmin)/2);
					AS.add(s9);
					// middle point for each edge
					Solution s10 =new Solution(NumberOfVar,NumberOfObj);
					s10.setObjective(0, (xmax+xmin)/2);
					s10.setObjective(1, ymin);
					s10.setObjective(2, zmin);
					AS.add(s10);
					Solution s11 =new Solution(NumberOfVar,NumberOfObj);
					s11.setObjective(0, xmax);
					s11.setObjective(1, (ymax+ymin)/2);
					s11.setObjective(2, zmin);
					AS.add(s11);
					Solution s12 =new Solution(NumberOfVar,NumberOfObj);
					s12.setObjective(0, (xmax+xmin)/2);
					s12.setObjective(1, ymax);
					s12.setObjective(2, zmin);
					AS.add(s12);
					Solution s13 =new Solution(NumberOfVar,NumberOfObj);
					s13.setObjective(0, xmin);
					s13.setObjective(1, (ymax+ymin)/2);
					s13.setObjective(2, zmin);
					AS.add(s13);
					Solution s14 =new Solution(NumberOfVar,NumberOfObj);
					s14.setObjective(0, xmax);
					s14.setObjective(1, ymin);
					s14.setObjective(2, (zmax+zmin)/2);
					AS.add(s14);
					Solution s15 =new Solution(NumberOfVar,NumberOfObj);
					s15.setObjective(0, xmax);
					s15.setObjective(1, ymax);
					s15.setObjective(2, (zmax+zmin)/2);
					AS.add(s15);
					Solution s16 =new Solution(NumberOfVar,NumberOfObj);
					s16.setObjective(0, xmin);
					s16.setObjective(1, ymax);
					s16.setObjective(2, (zmax+zmin)/2);
					AS.add(s16);
					Solution s17 =new Solution(NumberOfVar,NumberOfObj);
					s17.setObjective(0, xmin);
					s17.setObjective(1, ymin);
					s17.setObjective(2, (zmax+zmin)/2);
					AS.add(s17);
					Solution s18 =new Solution(NumberOfVar,NumberOfObj);
					s18.setObjective(0, (xmax+xmin)/2);
					s18.setObjective(1, ymin);
					s18.setObjective(2, zmax);
					AS.add(s18);
					Solution s19 =new Solution(NumberOfVar,NumberOfObj);
					s19.setObjective(0, xmax);
					s19.setObjective(1, (ymax+ymin)/2);
					s19.setObjective(2, zmax);
					AS.add(s19);
					Solution s20 =new Solution(NumberOfVar,NumberOfObj);
					s20.setObjective(0, (xmax+xmin)/2);
					s20.setObjective(1, ymax);
					s20.setObjective(2, zmax);
					AS.add(s20);
					Solution s21 =new Solution(NumberOfVar,NumberOfObj);
					s21.setObjective(0, xmin);
					s21.setObjective(1, (ymax+ymin)/2);
					s21.setObjective(2, zmax);
					AS.add(s21);
				}// for this region
			}
			AspirationSet=new NondominatedPopulation(AS);
	
			GenerateAspirationSets=false;
		}	
		// set initial values
		for(Solution s:population){
		s.setAttribute(FITNESS_ATTRIBUTE, 0.0);
		s.setAttribute(GD_ATTRIBUTE, 0.0);
		}
		// calculate Fitness attribute (delta1) and Generational Distance attribute (GD) 
		
		
		if (population.size() == 1) {
			for (Solution solution : population) {
				solution.setAttribute(FITNESS_ATTRIBUTE, 0.0);
				solution.setAttribute(GD_ATTRIBUTE, 0.0);
			}
		} else {
			// normalization using minimumBound and maximumBound

			List<Solution> solutions = Normalization(population);
			
			List<Solution> solutionsCopy = new ArrayList<Solution>(solutions);

			for (int i = 0; i < population.size(); i++) {
				solutionsCopy = new ArrayList<Solution>(solutions);
				solutionsCopy.remove(i);
				//
				//Calculate GD first
		        double sum1 = 0.0;
				int p=1;

				for (int j = 0; j < solutionsCopy.size(); j++) {
					sum1 += Math.pow(IndicatorUtils.distanceToNearestSolution(problem,
							solutionsCopy.get(j), AspirationSet), p);
				}
				
				double GD= Math.pow(sum1/ solutionsCopy.size(), 1.0 / p) ;
				population.get(i).setAttribute(GD_ATTRIBUTE, GD);
				
				//Calulate IGD then 
				double sum = 0.0;
				NondominatedPopulation NonSolution=new NondominatedPopulation(solutionsCopy);

				for (int k = 0; k < AspirationSet.size(); k++) {
					sum += Math.pow(IndicatorUtils.distanceToNearestSolution(problem,
							AspirationSet.get(k), NonSolution), p);
				}

				double IGD= Math.pow(sum/ AspirationSet.size(), 1.0 / p) ;
				
				double delta1= Math.max(GD, IGD);
								
				population.get(i).setAttribute(FITNESS_ATTRIBUTE, delta1);
				
			}

		}
		
	}

	private List<Solution> Normalization(Population solutions) {
     List<Solution> result = new ArrayList<Solution>();
		
		double[] min = minimumBound;
		double[] max = maximumBound;
				
		for (Solution solution : solutions) {
			Solution newSolution = solution.copy();
			
			if(solution.getAttribute(RANK_ATTRIBUTE)!=null)
			{
				newSolution.setAttribute(RANK_ATTRIBUTE, (int)solution.getAttribute(RANK_ATTRIBUTE));
			}
			
			
			for (int i = 0; i < problem.getNumberOfObjectives(); i++) {
			//	newSolution.setObjective(i, (newSolution.getObjective(i) - min[i]) / (max[i] - min[i]));
				newSolution.setObjective(i, newSolution.getObjective(i));
			}

			result.add(newSolution);
		}
		
		return result;
	}

	@Override
	public void evaluateT(Population population) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public boolean areLargerValuesPreferred() {
		// TODO Auto-generated method stub
		return true;
	}

	@Override
	public void evaluateDF(Population population) {
		// TODO Auto-generated method stub
		
	}

}
