package org.moeaframework.core.fitness;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.math.BigDecimal;

import org.apache.commons.math3.util.ArithmeticUtils;
import org.apache.commons.math3.util.CombinatoricsUtils;
import org.moeaframework.Executor;
import org.moeaframework.core.FitnessEvaluator;
import org.moeaframework.core.FrameworkException;
import org.moeaframework.core.Population;
import org.moeaframework.core.Problem;
import org.moeaframework.core.Settings;
import org.moeaframework.core.Solution;
import org.moeaframework.core.NondominatedSorting.preference;
import org.moeaframework.core.comparator.FitnessComparator1;
import org.moeaframework.core.comparator.FitnessComparator2;
import org.moeaframework.core.comparator.ObjectiveComparator;
import org.moeaframework.core.comparator.TchebycheffDistanceComparator1;
import org.moeaframework.core.comparator.TchebycheffDistanceComparator2;
import org.moeaframework.core.indicator.Hypervolume;



public class R2FitnessEvaluator implements FitnessEvaluator {
	
	/**
	 * The problem.
	 */
	private final Problem problem;
	//public static int change=0;
	private static boolean Generateweights=true;
	public static double[][] weight = null;
	public static int numberOfWeight = 0;
	
	public static final String RANK_ATTRIBUTE = "rank";
	public static final String CROWDING_NUMBER = "INDEX";
	
	public static final String DISTANCE_ATTRIBUTE = "TchebycheffDistance";
	public static final String SORT_ATTRIBUTE = "SortingCriterion"; 

	/**
	 * Constructs a new hypervolume contribution fitness evaluator.
	 * 
	 * @param problem the problem
	 * @param offset the offset used when determining the reference point for\
	 *               the hypervolume calculation.
	 */
	public R2FitnessEvaluator(Problem problem) {
		super();
		this.problem = problem;
	}
	
//	public static double evaluatepf(List<Solution> population) {
//		double[] minBound={0, 0};
//		double[] maxBound={1, 1};	
//		double[] referencePoint={0, 0};
//		int numberOfObjectives = population.get(0).getNumberOfObjectives();
//		
//		//List<Solution> solutions = coordinatetransformation(population, minBound, maxBound, referencePoint);
//		double totalVolume = Hypervolume.NewcalculateHypervolume(population, population.size(), numberOfObjectives, minBound, maxBound, referencePoint);
//		
//		return  totalVolume;
//	}

	
	@Override
	public void evaluate(Population population) {

		int numberOfObjectives = problem.getNumberOfObjectives();
		
		double stepsize = 0.0;
		
		if(Generateweights){

		// compute weights
		if (population.size() != 1) {
			
//			if (numberOfObjectives == 2) {
//				numberOfWeight = 500 + 1;
//				stepsize = (double) 1 / (numberOfWeight - 1);
//				// double weight[][]=new double
//				// [numberOfWeight][numberOfObjectives];
//				weight = new double[numberOfWeight][numberOfObjectives];
//				for (int k = 0; k < numberOfWeight; k++) {
//					weight[k][0] = k * stepsize;
//					weight[k][1] = 1 - k * stepsize;
//				}
//			} else if (numberOfObjectives == 3) {
//
//				numberOfWeight = 496;
//				stepsize = (double) 1 / 30;
//				weight = new double[numberOfWeight][numberOfObjectives];
//				int k = 0;
//				// int n = 30;
//				int count = 0;
//				while (k * stepsize <= 1) {
//					int i = 0;
//					if (k == 0) {
//						while (i * stepsize <= 1) {
//							weight[count][0] = k * stepsize;
//							weight[count][1] = i * stepsize;
//
//							weight[count][2] = 1 - weight[count][1] - weight[count][0];
//
//							i++;
//							count++;
//						}
//					} else {
//						while (i * stepsize < 1) {
//							weight[count][0] = k * stepsize;
//							weight[count][1] = i * stepsize;
//							if (weight[count][0] + weight[count][1] > 1) {
//								weight[count][1] = 1 - weight[count][0];
//							}
//							weight[count][2] = 1 - weight[count][1] - weight[count][0];
//
//							i++;
//
//							// if(weight[count][2]<=0 )
//							if ((double) (Math.round(weight[count][2] * 10000) / 10000.0) <= 0) {
//								weight[count][2] = 0;
//								count++;
//								break;
//							}
//							count++;
//						}
//					}
//					k++;
//				}
//			} else {
//				System.out.print("T-R2 only support 2 and 3 dimension problems now!");
//			}
			
			// use weights generation method in RIndicator.java
			int subdivisions=getDefaultSubdivisions(numberOfObjectives);
			weight = generateUniformWeights(subdivisions,
					numberOfObjectives);
			numberOfWeight=weight.length;
		}
		
		Generateweights=false;
		}

		if (population.size() == 1) {
			for (Solution solution : population) {
				solution.setAttribute(FITNESS_ATTRIBUTE, 0.0);
			}
		} else {

			List<Solution> solutions = coordinatetransformation1(population);
			List<Solution> solutionsCopy = new ArrayList<Solution>(solutions);

			for (int i = 0; i < population.size(); i++) {
				solutionsCopy = new ArrayList<Solution>(solutions);
				solutionsCopy.remove(i);
				double r2value = calculateR2(solutionsCopy, solutionsCopy.size(), numberOfObjectives, numberOfWeight,
						weight);
				population.get(i).setAttribute(FITNESS_ATTRIBUTE, r2value);
			}

		}
	}
	
	/**
	 * Generates uniformly-distributed weights.
	 * 
	 * @param s the number of subdivisions along each objective
	 * @param k the number of objectives
	 * @return the uniformly-distributed weights
	 */
	private static double[][] generateUniformWeights(int s, int k) {
		int counter = 0;
		int N = ArithmeticUtils.pow(s+1, k);
		
		double[][] weights = new double[
				(int)CombinatoricsUtils.binomialCoefficient(s+k-1, k-1)][k];
		
		for (int i = 0; i < N; i++) {
			int sum = 0;
			int[] kary = toBaseK(i, s+1, k);
			
			for (int j = 0; j < k; j++) {
				sum += kary[j];
			}
			
			if (sum == s) {
				for (int j = 0; j < k; j++) {
					weights[counter][j] = kary[j] / (double)s;
				}
				
				counter++;
			}
		}
		
		return weights;
	}

	/**
	 * Converts an integer into its base-k representation.
	 * 
	 * @param number the integer to convert
	 * @param k the base
	 * @param length the length of the resulting base-k representation
	 * @return the base-k representation of the given number
	 */
	private static int[] toBaseK(int number, int k, int length) {
		int value = length-1;
		int[] kary = new int[length];
		int i = 0;
		
		if (number >= ArithmeticUtils.pow(k, length)) {
			throw new FrameworkException("number can not be represented in " +
					"base-k with specified number of digits");
		}
		
		while (number != 0) {
			if (number >= ArithmeticUtils.pow(k, value)) {
				kary[i]++;
				number -= ArithmeticUtils.pow(k, value);
			} else {
				value--;
				i++;
			}
		}
		
		return kary;
	}
	
	/**
	 * Returns the default number of subdivisions for a given problem. The
	 * defaults, for an M objective problem, are:
	 * <ul>
	 *   <li>if M=2, then 500
	 *   <li>if M=3, then 30
	 *   <li>if M=4, then 12
	 *   <li>if M=5, then 8
	 *   <li>else 3
	 * </ul>
	 * 
	 * @param problem the problem
	 * @return the default number of subdivisions for a given problem
	 */
	public static int getDefaultSubdivisions(int NumberOfObj) {
		switch (NumberOfObj) {
		case 2:
			return 500;
		case 3:
			return 30;
		case 4:
			return 12;
		case 5:
			return 8;
		default:
			return 3;
		}
	}
	

	private List<Solution> coordinatetransformation1(Population population) {
		List<Solution> result = new ArrayList<Solution>();
		boolean isInverted = true;
		boolean isCustomHypervolume = (Settings.getHypervolume() != null) && 
				(population.get(0).getNumberOfObjectives() > 2) ;
		
		if (isCustomHypervolume) {
			isInverted = Settings.isHypervolumeInverted();
		}

		
		for (Solution solution : population) {
			Solution newSolution = solution.copy();

			
			for (int i = 0; i < solution.getNumberOfObjectives(); i++)
			{
				if (!isInverted) 
				{
					newSolution.setObjective(i, -solution.getObjective(i));
				}
			}
			result.add(newSolution);
		}
		
		return result;
	}
	
// it takes 0 as ideal point for each objective, for other situations, the codes have to been modified
	public static double calculateR2(List<Solution> population, int numberOfSolutions, 
			int numberOfObjectives, int numberOfWeight, double [][] weight) {
		
		double r2indicator = 0.0;
		

		
		for (int k = 0; k< numberOfWeight; k++)
		{
			double disSolution = 0.0;
			for (int i = 0; i < numberOfSolutions; i++) 
			{
				
				double dis = 0.0;
				for(int j = 0; j < numberOfObjectives; j++) 
				{
					double Td=population.get(i).getObjective(j) * weight[k][j];
					
					if (Td > dis)
					{
						dis = Td;
					}
				}
				if (i == 0)
				{
					disSolution = dis;
				}
				else if (disSolution > dis)
				{
					disSolution = dis;
				}
			}
			
//			BigDecimal bd1 = new BigDecimal(Double.toString(r2indicator)); 
//	        BigDecimal bd2 = new BigDecimal(Double.toString(disSolution)); 
//	        r2indicator = bd1.add(bd2).doubleValue(); 
			r2indicator = r2indicator + disSolution;
		}
		
//		int scale = 6;
//		BigDecimal bd1 = new BigDecimal(Double.toString(r2indicator)); 
//        BigDecimal bd2 = new BigDecimal(Double.toString(numberOfWeight)); 
//        return bd1.divide (bd2,scale,BigDecimal.ROUND_HALF_UP).doubleValue(); 
		return (double)(r2indicator/numberOfWeight);
	}
	
	
	public static Solution coordinatetransformationCT(Solution solution,int numberOfObjectives, double[] centerPoint, double radius) {
		
		Solution newSolution = solution.copy();
		boolean isInverted = true;
		boolean isCustomHypervolume = (Settings.getHypervolume() != null) && (numberOfObjectives > 2) ;
		
		if (isCustomHypervolume) {
			isInverted = Settings.isHypervolumeInverted();
		}
		
		int tempindex = (int)solution.getAttribute(INDEX_ATTRIBUTE);
		newSolution.setAttribute(INDEX_ATTRIBUTE, tempindex);
		
		if (isInverted)
		{
			for (int i=0;i<numberOfObjectives;i++)
			{
				
				newSolution.setObjective(i, newSolution.getObjective(i)-(centerPoint[i]-radius));	
			}	
		}
		else
		{
			for (int i=0;i<numberOfObjectives;i++)
			{

				newSolution.setObjective(i, 2*centerPoint[i]-newSolution.getObjective(i)-(centerPoint[i]-radius));		
				
				//newSolution.setObjective(i, 2*centerPoint[i]-newSolution.getObjective(i));					
			}
		}

				
		return newSolution;
	}
	
	
public static Solution coordinatetransformationRT(Solution solution,double[] minBound, double[] maxBound) {
		
		Solution newSolution = solution.copy();
		
		
			int tempindex = (int)solution.getAttribute(CROWDING_NUMBER);
			newSolution.setAttribute(CROWDING_NUMBER, tempindex);
			double tempdis = (double)solution.getAttribute(DISTANCE_ATTRIBUTE);
			newSolution.setAttribute(DISTANCE_ATTRIBUTE, tempdis);
//			double tempCdis = (double)solution.getAttribute(CROWDING_ATTRIBUTE);
//			newSolution.setAttribute(CROWDING_ATTRIBUTE, tempCdis);
			int rank= (int) solution.getAttribute(RANK_ATTRIBUTE);
			newSolution.setAttribute(RANK_ATTRIBUTE, rank);
			
			for (int i = 0; i < solution.getNumberOfObjectives(); i++)
			{
				newSolution.setObjective(i, TRFunction(solution.getObjective(i), i, minBound, maxBound));
			}
					
		return newSolution;
	}


	
	public static List<Solution> coordinatetransformation(Population population,double[] minBound, double[] maxBound) {
		
		List<Solution> result = new ArrayList<Solution>();
		boolean isInverted = true;
		boolean isCustomHypervolume = (Settings.getHypervolume() != null) && 
				(population.get(0).getNumberOfObjectives() > 2) ;
		
		if (isCustomHypervolume) {
			isInverted = Settings.isHypervolumeInverted();
		}

		
		for (Solution solution : population) {
			Solution newSolution = solution.copy();
			
			int tempindex = (int)solution.getAttribute(INDEX_ATTRIBUTE);
			newSolution.setAttribute(INDEX_ATTRIBUTE, tempindex);
			
			for (int i = 0; i < solution.getNumberOfObjectives(); i++)
			{
				if (isInverted) 
				{
					newSolution.setObjective(i, TRFunction(solution.getObjective(i), i, minBound, maxBound));
				}
				else
				{
					newSolution.setObjective(i, TRFunctionReverse(solution.getObjective(i), i, minBound, maxBound));
				}
			}
			result.add(newSolution);
		}
		
		return result;
	}
	
	public static double TRFunction(double y, int objective, double[] minBound, double[] maxBound){
		double result=0;
		
		if (y <= minBound[objective])
		{
			result = 0;
		}
		else if(y < maxBound[objective])
		{
			result = y - minBound[objective];
		}
		else
		{
			result = maxBound[objective] - minBound[objective];
		}
	
		return result;
	}
	
	public static double TRFunctionReverse(double y, int objective, double[] minBound, double[] maxBound){
		double result=0;
		
		if (y >= maxBound[objective])
		{
			result = 0;
		}
		else if(y > minBound[objective])
		{
			result = maxBound[objective]-y;
		}
		else
		{
			result = maxBound[objective] - minBound[objective];
		}
	
		return result;
	}
	
	@Override
	public boolean areLargerValuesPreferred() {
		return true;
	}

	@Override
	public void evaluateT(Population population) {
		boolean checkSolutionInROIwithoutattribute=false;
		int numberOfSolutionsInFront=population.size();
		int numberOfTRegion =preference.NumOfROI;
		int numberOfObjectives = population.get(0).getNumberOfObjectives();
		String Shape=preference.shape;
		
	//compute weights		
		
		double stepsize = 0.0;
		
		if(Generateweights)
		{
		if (population.size() != 1) 
		{
//			if (numberOfObjectives == 2)
//			{
//				numberOfWeight = 500+1;
//				stepsize = (double)1/(numberOfWeight-1);
//				//double weight[][]=new double [numberOfWeight][numberOfObjectives];
//				weight = new double [numberOfWeight][numberOfObjectives];
//				for (int k = 0; k< numberOfWeight; k++)
//				{
//					weight[k][0] = k*stepsize;
//
//					weight[k][1] = 1-k*stepsize;
//				}
//			}
//			else if (numberOfObjectives == 3)
//			{
//			
//				numberOfWeight = 496;
//				stepsize = (double)1/30;
//				weight = new double [numberOfWeight][numberOfObjectives];
//				int k = 0;
//				//int n = 30;
//				int count = 0;
//				while (k*stepsize<=1)
//				{
//					int i = 0;
//					if (k==0)
//					{
//						while (i*stepsize<=1)
//						{
//							weight[count][0] = k*stepsize;
//							weight[count][1] = i*stepsize;
//							
//							weight[count][2] = 1-weight[count][1]-weight[count][0];
//			
//							i++;
//							count++;
//						}
//					}
//					else
//					{
//						while (i*stepsize<1)
//						{
//							weight[count][0] =k*stepsize;
//							weight[count][1] = i*stepsize;
//							if (weight[count][0] + weight[count][1]>1)
//							{
//								weight[count][1] = 1-weight[count][0];
//							}
//							weight[count][2] = 1-weight[count][1]-weight[count][0];
//							
//							i++;
//							
//							//if(weight[count][2]<=0 )
//							if ((double) (Math.round(weight[count][2]*10000)/10000.0)<=0)
//							{
//								weight[count][2] = 0;
//								count++;
//								break;
//							}
//							count++;
//						}
//					}
//					k++;
//				}	
//			}
//			else
//			{
//				System.out.print("T-R2 only support 2 and 3 dimension problems now!");
//			}
			
			
			// use weights generation method in RIndicator.java
			int subdivisions=getDefaultSubdivisions(numberOfObjectives);
			weight = generateUniformWeights(subdivisions,
					numberOfObjectives);
			numberOfWeight=weight.length;
			
		}
		Generateweights=false;
		} // weight generation end
		
		if(Shape.equalsIgnoreCase("point")){
			for(Solution s:population)
			{
				s.setAttribute(FITNESS_ATTRIBUTE, 0.0);
			}
		} // if reference point is used, no need to calculate fitness attribute
		else{
		List<Double[]> Targets =new ArrayList<Double[]>();	
		Double[][] TargetRegions=preference.regions; // the array is numberOfTRegion * (numberOfObjectives+1) for circles
		Double[][] CenterPoints= new Double[numberOfTRegion][numberOfObjectives]; //Array of CenterPoints
		 Map <Integer, double[]> AttributeMap=new HashMap<Integer, double[]>();// double[numberOfTRegion] to record delta-HV in each ROI
		

	//	AttributeMap	initialization
		for(Solution s:population)
		{
			double[] HVAttribute=new double[numberOfTRegion];
			for(int t=0;t<numberOfTRegion;t++){
				HVAttribute[t]=0;
			}
			AttributeMap.put((int)s.getAttribute(CROWDING_NUMBER), HVAttribute);
			s.setAttribute(FITNESS_ATTRIBUTE, 0.0);
		}
	// CenterPoints	
		for(int i=0;i<numberOfTRegion;i++)
		{
			switch(Shape){
			case "circle":
				Double[] R=new Double[numberOfObjectives+1];
				for(int j=0;j<numberOfObjectives+1;j++){
					R[j]=TargetRegions[i][j];
					if(j!=numberOfObjectives)
					{
						CenterPoints[i][j]=TargetRegions[i][j];
					}
				}
				Targets.add(R);
				break;
			case "rectangle":
				Double[] R1=new Double[numberOfObjectives*2];
				for(int j=0;j<numberOfObjectives*2;j++)
					{R1[j]=TargetRegions[i][j];}
				
				Targets.add(R1);

				break;
//			case "point":	
//				Double[] R2=new Double[numberOfObjectives];
//				for(int j=0;j<numberOfObjectives;j++)
//				{R2[j]=TargetRegions[i][j];
//				CenterPoints[i][j]=TargetRegions[i][j];}
//				Targets.add(R2);
//				break;
			default:
	             throw new IllegalArgumentException("Invalid shape of preference region: " + Shape);
			}
			
			
		}
		
		// set CenterPoint for rectangles.
		if (Shape=="rectangle")
	{
			for(int i=0;i<numberOfTRegion;i++)
			{
				
				for(int j=0;j<numberOfObjectives;j++)
					
				{
					CenterPoints[i][j]=(TargetRegions[i][j]+TargetRegions[i][j+numberOfObjectives])/2;
				}
				
				
			}
									
	}
					
	
		
		if (population.size() == 1) {
			
	//	population.get(0).setAttribute(FITNESS_ATTRIBUTE, 0.0);
		population.get(0).setAttribute(FITNESS_ATTRIBUTE, (double)numberOfSolutionsInFront);
			
		} else {
			double[] MaxR2Contribution= new double[numberOfTRegion];
//			find solutions in the ROIs
			for(int t=0;t<numberOfTRegion;t++){
				
				List<Solution> tempfront = new ArrayList<Solution>();
				// find solutions in this region
					for(Solution s:population){
						if(Shape=="circle"){
							double distance=0.0;
							double[] centerPoint=new double[numberOfObjectives];
							double radius=TargetRegions[t][numberOfObjectives];
							for (int i=0;i<numberOfObjectives;i++)
							{
								distance+=Math.pow(Math.abs(s.getObjective(i)- CenterPoints[t][i]),2);
								centerPoint[i]=CenterPoints[t][i];
							}
							if (distance <= Math.pow(radius, 2))// in the region
							{
								Solution solutiontemp = coordinatetransformationCT(s, numberOfObjectives, centerPoint, radius);
								tempfront.add(solutiontemp);
							}
						
						}
						else if(Shape=="rectangle"){
							double[] minBound=new double[numberOfObjectives];
							double[] maxBound=new double[numberOfObjectives];
							boolean InROI=true;
							for (int i=0;i<numberOfObjectives;i++)
							{
								minBound[i]=TargetRegions[t][i];
								maxBound[i]=TargetRegions[t][i+numberOfObjectives];
								if((s.getObjective(i)>maxBound[i])||(s.getObjective(i)<minBound[i])){
									InROI=false;
									break;
								}
							}
							if(InROI){
								Solution solutiontemp=coordinatetransformationRT( s,  minBound, maxBound);
								tempfront.add(solutiontemp);
							}
						
					}
					
					
				}
		if(tempfront.size()>0){// there exist solutions in ROI
		   checkSolutionInROIwithoutattribute=true;

			List<Solution> solutionsCopy = new ArrayList<Solution>(tempfront);

			if (solutionsCopy.size()==1)
			{
				// set fitness as a large value
				int index=(int) solutionsCopy.get(0).getAttribute(INDEX_ATTRIBUTE);
				population.get(index).setAttribute(FITNESS_ATTRIBUTE, (double)numberOfSolutionsInFront);
//				int tempindex = (int) tempfront.get(0).getAttribute(INDEX_ATTRIBUTE);
//				double[] R2temp=AttributeMap.get(tempindex);
//				R2temp[t]=0.0;
			}
	
			else if(solutionsCopy.size()>1)
			{
				for (int i = 0; i < tempfront.size(); i++) {
					solutionsCopy = new ArrayList<Solution>(tempfront);	
					solutionsCopy.remove(i);
					double r2value = calculateR2(solutionsCopy, solutionsCopy.size(), numberOfObjectives, numberOfWeight, weight);
					
					int tempindex = (int) tempfront.get(i).getAttribute(INDEX_ATTRIBUTE);
					double[] R2temp=AttributeMap.get(tempindex);
					R2temp[t]=r2value;
	
				}	
			
			// set Fitness
			for(Solution solution : population){
				int index= (int) solution.getAttribute(CROWDING_NUMBER);
				double[] AttributeForthisSolution=AttributeMap.get(index);
				if(AttributeForthisSolution!=null)
				solution.setAttribute(SORT_ATTRIBUTE, AttributeForthisSolution[t]);
			}
			
			population.sort(new TchebycheffDistanceComparator1());// from 0 to infinity	
			double maxR2 = (double)population.get(numberOfSolutionsInFront-1).getAttribute(SORT_ATTRIBUTE);
			MaxR2Contribution[t]=maxR2;
			
			int numdis = 0;
			
			for (int m=numberOfSolutionsInFront-1;m>=0;m--) {
					
				if (numdis <= Math.floor(numberOfSolutionsInFront/numberOfTRegion)) 
				{
					if (((double)population.get(m).getAttribute(FITNESS_ATTRIBUTE)<(double)(numberOfSolutionsInFront-numdis))&& ((double)population.get(m).getAttribute(SORT_ATTRIBUTE)!=0.0))
					
					{
						population.get(m).setAttribute(FITNESS_ATTRIBUTE, (double)(numberOfSolutionsInFront+1-numdis));
					}
					numdis = numdis + 1;
				}
			}
			
			} // the number of solutions in ROI is more than 1
		}// the number of solutions in ROI is more than 0
	} // for number of targets
			
			// handle solutions without Fitness_ATTRIBUTE for more than two ROI
			if((numberOfTRegion>1)&&checkSolutionInROIwithoutattribute){
			for (Solution solution : population) {
				double tempdata= (double)solution.getAttribute(FITNESS_ATTRIBUTE);
				int index= (int) solution.getAttribute(CROWDING_NUMBER);
				double[] AttributeForthisSolution=AttributeMap.get(index);
				double[] crowdingdata=new double[numberOfTRegion];
				for(int i=0;i<numberOfTRegion;i++){
					crowdingdata[i]=AttributeForthisSolution[i];
				}

				if ((tempdata == 0.0) && (HasNoneZeroValue(crowdingdata)))
				{
				 double[] newdata= new double[numberOfTRegion];
					for(int k=0;k<numberOfTRegion;k++){
						double tempdata3 = 0.0;
						if ((crowdingdata[k] != 0.0) && (MaxR2Contribution[k] !=0.0))
						{
							
						//	tempdata3 = crowdingdata[k];
							// will it be more than 1 ?
							tempdata3 = crowdingdata[k]/MaxR2Contribution[k];
							newdata[k]=tempdata3;
						}
					}
					double value=FindLargestInDoubleArray(newdata);
//					double minvalue=FindSmallestInDoubleArray(newdata);
					solution.setAttribute(FITNESS_ATTRIBUTE, value);	
				}
				
			}
			}
		} // population size >1
		}// reference point is not used, region instead
}
	private double FindLargestInDoubleArray(double[] newdata) {
		 // Validates input
	    if (newdata == null) {
	        throw new IllegalArgumentException("The Array must not be null");
	    } else if (newdata.length == 0) {
	        throw new IllegalArgumentException("Array cannot be empty.");
	    }

	    // Finds and returns max
	    double max = newdata[0];
	    for (int i = 1; i < newdata.length; i++) {
	        if (Double.isNaN(newdata[i])) {
	            return Double.NaN;
	        }
	        if (newdata[i] > max) {
	       	 max = newdata[i];
	        }
	    }

	    return max;
	}

	private boolean HasNoneZeroValue(double[] crowdingdata) {
		// TODO at least one of the elements in crowdingdata is not 0
		boolean redult=false;
		int num=crowdingdata.length;
		for(int i=0;i<num;i++){
			if(crowdingdata[i]!=0.0)
			{
				redult=true;
				break;
			}
		}
		return redult;
	}

	@Override
	public void evaluateDF(Population population) {
		// TODO Auto-generated method stub
		
	}
	
}


