package org.moeaframework.core.indicator;

import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.Problem;
import org.moeaframework.core.Settings;
import org.moeaframework.core.Solution;

public class AveragedHausdorffDistance extends NormalizedIndicator {
	
	/**
	 * can be set to 1.2.or positive infinity
	 * 
	 */
	private static double p=1;
	
	//Bound of deltap for preference based 
	private static double[] minimumBound;
	private static double[] maximumBound;
	//whether the preference based indicator is used
	private static boolean PAHD;
	
	public AveragedHausdorffDistance(Problem problem,
			NondominatedPopulation referenceSet) {
		this(problem, referenceSet, p);
		
		
	}
	
	public AveragedHausdorffDistance(Problem problem,
			NondominatedPopulation referenceSet,double[]idealPoint,double[]NadirPoint, double[] min, double[] max) {
		
		this(problem, referenceSet,idealPoint,NadirPoint, p);
		minimumBound=min;
		maximumBound=max;
		PAHD=true;
	}
	

	public AveragedHausdorffDistance(Problem problem,
			NondominatedPopulation referenceSet,
			double d) {
		super(problem, referenceSet);
		AveragedHausdorffDistance.p = d;
	}
	
	public AveragedHausdorffDistance(Problem problem,
			NondominatedPopulation referenceSet,double[]idealPoint,double[]NadirPoint,
			double d) {
		super(problem, referenceSet,idealPoint,NadirPoint);
		AveragedHausdorffDistance.p = d;
	}
	

	@Override
	public double evaluate(NondominatedPopulation approximationSet) {
		if(PAHD){
			NondominatedPopulation approximateSetINROI=FilterinROI(approximationSet);
			
			NondominatedPopulation referenceSet=getNormalizedReferenceSet();
			
			//Calculate GD first
	        double sum1 = 0.0;
			
			if (approximateSetINROI.isEmpty()) {
				return Double.POSITIVE_INFINITY;
			}

			for (int i = 0; i < approximateSetINROI.size(); i++) {
				sum1 += Math.pow(IndicatorUtils.distanceToNearestSolution(problem,
						approximateSetINROI.get(i), referenceSet), p);
			}
			
			Double GD= Math.pow(sum1/ approximateSetINROI.size(), 1.0 / p) ;
			
			//Calulate IGD then 
			double sum = 0.0;

			for (int i = 0; i < referenceSet.size(); i++) {
				sum += Math.pow(IndicatorUtils.distanceToNearestSolution(problem,
						referenceSet.get(i), approximateSetINROI), p);
			}

			Double IGD= Math.pow(sum/ referenceSet.size(), 1.0 / p) ;
			
			return Math.max(GD, IGD);

		}
		else{
		NondominatedPopulation referenceSet=getNormalizedReferenceSet();
		
		NondominatedPopulation np=normalize(approximationSet);
		//Calculate GD first
        double sum1 = 0.0;
		
		if (np.isEmpty()) {
			return Double.POSITIVE_INFINITY;
		}

		for (int i = 0; i < np.size(); i++) {
			sum1 += Math.pow(IndicatorUtils.distanceToNearestSolution(problem,
					np.get(i), referenceSet), p);
		}
		
		Double GD= Math.pow(sum1/ np.size(), 1.0 / p) ;
		
		//Calulate IGD then 
		double sum = 0.0;

		for (int i = 0; i < referenceSet.size(); i++) {
			sum += Math.pow(IndicatorUtils.distanceToNearestSolution(problem,
					referenceSet.get(i), np), p);
		}

		Double IGD= Math.pow(sum/ referenceSet.size(), 1.0 / p) ;
		
		return Math.max(GD, IGD);
		}
		
	}

	private NondominatedPopulation FilterinROI(NondominatedPopulation approximationSet) {
		
		NondominatedPopulation result=new NondominatedPopulation();
		
		for (Solution solution : approximationSet) {
			//prune any solutions which exceed the Bound
			boolean Add=true;
			for (int i=0; i<solution.getNumberOfObjectives(); i++) {
				
					if(solution.getObjective(i)>maximumBound[i]||solution.getObjective(i)<minimumBound[i]){
						Add=false;
						break;
					}
				}
			
			if(Add){
			Solution clone = solution.copy();
					
			result.add(clone);		
			}
		}
		
		return result;
	}
	
}
