package org.moeaframework.core.variable;

//import java.util.BitSet;

//import org.moeaframework.core.PRNG;
import org.moeaframework.core.Variable;

/**
 * Decision variable for Double array.
 */

public class DoubleArrayVariable implements Variable{

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The number of double stored in this variable.
	 */
	private final int numberOfDouble;

	/**
	 * The internal storage for the double ARRAY.
	 */
	private final double[] doubleArray;
	
	
	/**
	 * Constructs a double array variable with the specified number of doubles. 
	 * 
	 * @param numberOfBits the number of bits stored in this variable
	 */
	public DoubleArrayVariable(int numberOfDouble) {
		super();
		this.numberOfDouble = numberOfDouble;

		doubleArray = new double[numberOfDouble];
		
	}
	
	public DoubleArrayVariable(double[] values) {
		super();
		this.numberOfDouble = values.length;

		doubleArray = new double[values.length];
		for(int i=0;i<values.length;i++)
		{
			doubleArray[i]=values[i];
		}
	}
	
	/**
	 * Returns the number of doubles stored in this variable.
	 * 
	 * @return the number of doubles stored in this variable
	 */
	public int getNumberOfBits() {
		return numberOfDouble;
	}
	
	/**
	 * Returns the value of the bit at the specified index.
	 * 
	 * @param index the index of the bit to return
	 * @return the value of the bit at the specified index
	 * @throws IndexOutOfBoundsException if the index is out of bounds
	 *         {@code (index < 0) || (index >= getNumberOfBits())}
	 */
	public double get(int index) {
		if ((index < 0) || (index >= numberOfDouble)) {
			throw new IndexOutOfBoundsException();
		}

		return doubleArray[index];
	}
	
	
	/**
	 * Set the value of the bit at the specified index.
	 * 
	 * @param index the index of the bit to set
	 * @param value the new value of the bit being set
	 * @throws IndexOutOfBoundsException if the index is out of bounds
	 *         {@code (index < 0) || (index >= getNumberOfBits())}
	 */
	public void set(int index, double value) {
		if ((index < 0) || (index >= numberOfDouble)) {
			throw new IndexOutOfBoundsException();
		}

		doubleArray[index]=value;
	}
	
	public void setArray(double[] value) {
		if (value.length!= numberOfDouble) {
			throw new IndexOutOfBoundsException();
		}
        for(int i=0;i<numberOfDouble;i++)
		doubleArray[i]=value[i];
	}
	
	
	/**
	 * Returns a {@link BitSet} representing the state of this variable.
	 * 
	 * @return a {@code BitSet} representing the state of this variable
	 */
	public double[] getArray() {
		return doubleArray;
	}
	
	@Override
	public void randomize() {
		for (int i = 0; i < getNumberOfBits(); i++) {
			set(i, 0.5);
		}
	}

	@Override
	public Variable copy() {
		// TODO Auto-generated method stub
		DoubleArrayVariable copy = new DoubleArrayVariable(numberOfDouble);
		copy.setArray(doubleArray);
		return copy;
		
	}
	
	
}
